from popenx import *
from select import *
from socket import *
import sys, os, signal, string, time

shellAccessHandler = []
def runCommand(server, pid, data):
	return "0,"+str(pid)+","+str(server.command(data))+"\0"
shellAccessHandler.append(runCommand)		## code 0

def kill(server, pid, data):
	return "1,"+str(pid)+","+str(server.kill(pid))+"\0"
shellAccessHandler.append(kill) ## code 1

def getStatus(server, pid, data):
	return "2,"+str(pid)+","+str(server.getStatus(pid))+"\0"
shellAccessHandler.append(getStatus) ## code 2

def writeInput(server, pid, data):
	return "3,"+str(pid)+","+str(server.writeInput(pid, data))+"\0"
shellAccessHandler.append(writeInput)		##code 3

def	outputWaiting(server, pid, data):
	return "4,"+str(pid)+","+str(server.outputWaiting(pid))+"\0"
shellAccessHandler.append(outputWaiting)		## code 4

def readOutput(server, pid, data):
	output = server.readOutput(pid, bytes =900)
	if output == None:
		return "5,"+str(pid)+","+'\1\2\2\1'+"\0"
	else:
		return "5,"+str(pid)+","+output+"\0"
shellAccessHandler.append(readOutput)			##code 5

def getLastError(server, pid, data):
	return "6,0,"+str(server.getLastError())+"\0"
shellAccessHandler.append(readOutput)			##code 6

def newError(server, pid, data):
	return "7,0,"+str(server.newError())+"\0"
shellAccessHandler.append(newError)				##code 7

def getCommands(server, pid, data):
	commandsDict = server.getCommands()
	keyList = commandsDict.keys()
	dataStream = ''
	for key in keyList:
		dataStream = dataStream + str(key) + '\1' + commandsDict[key] + '\2'
	return "8,0,"+dataStream+"\0"
shellAccessHandler.append(getCommands)			## code 8

def	setAutoWriteBack(server, pid, data):
	data.append(pid)
	return "9,"+str(pid)+","+str(pid)+"\0"
shellAccessHandler.append(setAutoWriteBack)								## code 9

def clearFinishedCommands(server, pid, data):
	return "10,0,"+str(server.clearFinishedCommands())+"\0"
shellAccessHandler.append(clearFinishedCommands)		## code 10

class ShellAccessServer:
	def __init__(self):
		self.commandDict = {}
		self.commandTextDict = {}
		self.lastError = None
		self.lastErrorChecked = 1

	def command(self, cmd):
		try:
			tmpCmd = PopenX1(cmd)
			self.commandDict[tmpCmd.pid] = tmpCmd
			self.commandTextDict[tmpCmd.pid] = cmd
			return tmpCmd.pid
		except Exception, e:
			self.lastErrorChecked = 0
			self.lastError = e
			return -2

	def kill(self, pid):
		try:
			os.kill(pid,signal.SIGTERM)
			return 1
		except Exception, e:
			self.lastErrorChecked = 0
			self.lastError = e
			return -2

	def getStatus(self, pid):
		try:
			return self.commandDict[pid].poll()
		except Exception, e:
			self.lastErrorChecked = 0
			self.lastError = e
			return -2

	def writeInput(self, pid, data):
		try:
			inputObject = self.commandDict[pid].tochild
			inputObject.write(data)
			inputObject.flush()
			return 1
		except Exception, e:
			self.lastErrorChecked = 0
			self.lastError = e
			return -2

	def outputWaiting(self, pid):
		try:
			output_fd = self.commandDict[pid].fromchild.fileno()
			readyList, d1, d2 = select([output_fd],[],[],0)
			if len(readyList) > 0:
				return 1
			else:
				return 0
		except Exception, e:
			self.lastErrorChecked = 0
			self.lastError = e
			return -2

	def readOutput(self, pid, bytes=1000):
		try:
			output_fd = self.commandDict[pid].fromchild.fileno()
			readyList, d1, d2 = select([output_fd],[],[],0)
			if len(readyList) > 0:
				output = os.read(output_fd, bytes)
				return output
			else:
				return ''
		except Exception, e:
			self.lastErrorChecked = 0
			self.lastError = e
			return None

	def getLastError(self):
		self.lastErrorChecked = 1
		return self.lastError

	def newError(self):
		return not self.lastErrorChecked

	def getCommands(self):
		return self.commandTextDict

	def clearFinishedCommands(self):
		keyList = self.commandDict.keys()
		delCount = 0
		for key in keyList:
			if self.commandDict[key].poll() != -1:
				del self.commandDict[key]
				del self.commandTextDict[key]
				delCount = delCount + 1
		return delCount

class ShellProxServer:
	__verbHandler = []

	def noDef(self, data):
		return (0, data)
	__verbHandler.append(noDef)

	def killVerb(self, data):
		self.kill()
		self.__output.write("1,!"+"\0")		##ack!
		self.__output.flush()
		return (1,None)
	__verbHandler.append(killVerb)

	def getPidVerb(self, data):
		data = str(self.__pid)
		self.__output.write("2,"+data+"\0")
		self.__output.flush()
		return (2,data)
	__verbHandler.append(getPidVerb)

	def	getStatusVerb(self, data):
		if self.__sPS == None:
			self.__output.write("3,-2"+"\0")
			self.__output.flush()
			return (3,-2)
		else:
			status = self.__sPS.poll()
			self.__output.write("3,"+str(status)+"\0")
			self.__output.flush()
			return (3,status)
	__verbHandler.append(getStatusVerb)

	def userInputVerb(self, data):
		return (4,data)
	__verbHandler.append(userInputVerb)

	def	__verbHandlerMethod(self, input):
		try:
			sepIndex = string.index(input,",")
			inputList = (int(input[:sepIndex]),input[sepIndex+1:])
			return self.__verbHandler[int(inputList[0])](self,inputList[1])
		except:
			return self.__verbHandler[0](self,input)

	def __init__(self, input = sys.stdin, output = sys.stdout):
		self.__output = output
		self.__input = input
		self.__pid = -1
		self.__sPS = None

	def timedRead(self, fileDescriptor, timer = 0, buffer = 1000):
		l1, l2, l3 = select([fileDescriptor],[],[],timer)
		if len(l1) != 0:
			try:
				data = os.read(fileDescriptor.fileno(),buffer)
				return (1, data, len(data))
			except Exception, e:
				return (-1, 'Error: '+str(e), 0)
		else:
			return (0, '', 0)
	
	def timedWrite(self, fileDescriptor, timer = 0, buffer = 1000):
		l1, l2, l3 = select([self.__input],[],[],timer)
		if len(l1) != 0:
			data = os.read(self.__input.fileno(), buffer)
			if len(data) == 0:
				self.kill()			## Socket broken
			messageList = string.split(data,"\0")
			for message in messageList:
				dataList = self.__verbHandlerMethod(message)
				if dataList[0] == 4:
					fileDescriptor.write(dataList[1]+"\n")
					fileDescriptor.flush()
			return 1
		else:
			return 0

	def kill(self):
		if self.__pid != -1:
			try:
				os.kill(self.__pid, signal.SIGTERM)
			except:
				pass

	def isAlive(self):
		if self.__sPS.poll() == -1:
			return 1
		else:
			return 0
	
	def shell(self, script):
		shellProxServ = PopenX1(script)
		self.__sPS = shellProxServ
		shellOut = shellProxServ.fromchild
		shellIn = shellProxServ.tochild
		self.__pid = shellProxServ.pid
		close = 0
		while not close:
			try:
				dataInfo = self.timedRead(shellOut,.5)
			except Exception,e:
				self.kill()
				dataInfo = [0,'',0]
			if dataInfo[0] != 1 and shellProxServ.poll() != -1:
				try:
					#print 'closing', dataInfo[0], shellProxServ.poll()
					self.__output.write("5,SHELL finished."+'\0')
					self.__output.flush()
					self.getPidVerb(None)
					self.getStatusVerb(None)
				except Exception, e:
					#print e
					pass
				close = 1
			if dataInfo[0] == 1:# and dataInfo[2] > 0:
				try:
					self.__output.write("4,"+dataInfo[1]+'\0')
					self.__output.flush()
				except:
					pass
			elif dataInfo[0] == -1:				##error
				self.kill()
			try:
				self.timedWrite(shellIn)
			except Exception, e:
				self.kill()

handleBreakArgsList = []
def handleBreak(signum, frame):
	global handleBreakArgsList
	if signum != signal.SIGINT:
		#print 'what?'
		sys.exit()
	else:
		#print 'cntrl-c on', os.getpid()
		#print 'closing ',handleBreakArgsList[0]
		handleBreakArgsList[0].close()
		handleBreakArgsList[1]()

if __name__=="__main__":
	global handleBreakArgsList
	try:
		portIndex = sys.argv.index('-port') + 1
		port = int(sys.argv[portIndex])
	except:
		port = 49999	#Default Port = 49999
	server = socket(AF_INET, SOCK_STREAM)
	handleBreakArgsList.append(server)
	handleBreakArgsList.append(signal.getsignal(signal.SIGINT))
	signal.signal(signal.SIGINT,handleBreak)
	server.bind(('',port))
	server.listen(10)
	alive = 1
	deadThreadList = []
	while alive:
		#print 'reset'
		readyInList = []
		while len(readyInList) == 0:
			readyInList, blankList, blankList = select([server],[],[],1)
			for index in range(0,len(deadThreadList)):
				process = deadThreadList[index][0]
				donePid, status = os.waitpid(process, os.WNOHANG)
				if donePid == process:
					#print 'deamon killing child',donePid
					deadThreadList[index][1].close()
					deadThreadList[index] = -1
			while -1 in deadThreadList:
				deadThreadList.remove(-1)
		#print 'forking'
		peer = server.accept()[0]
		childPid = os.fork()
		if childPid == 0:		## child
			#peer = server.accept()[0]
			shellArgs = peer.recv(1000)
			if shellArgs == "\1,STAY_ALIVE_SERVER\0":
				#print 'starting stay alive server'
				peer.send("\1,ACK\0")
				shellAccess = ShellAccessServer()
				commandDataDump = peer.recv(1000)
				done = 0
				autoPidWriteBackList = []
				while not done:
					commandDataList = string.split(commandDataDump,"\0")
					for commandData in commandDataList:
						if len(commandData) == 0:
							continue
						if commandData == "\1,SHUT_DOWN_SOCKET":
							done = 1
							peer.send("\1,ACK\0")
							break
						indexTemp = string.index(commandData,",")
						type = int(commandData[:indexTemp])
						commandData = commandData[indexTemp+1:]
						indexTemp = string.index(commandData,",")
						pid = int(commandData[:indexTemp])
						data = commandData[indexTemp+1:]
						## this is a hack:
						if type == 9:
							data = autoPidWriteBackList
						## End hack
						returnValue = shellAccessHandler[type](shellAccess,pid,data)
						peer.send(returnValue)
						for pid in autoPidWriteBackList:
							if shellAccess.outputWaiting(pid):
								output = shellAccess.readOutput(pid)
								if output != None and len(output) > 0:
									peer.send("5,"+str(pid)+","+output+"\0")
					marked = []
					for pid in autoPidWriteBackList:
						if shellAccess.outputWaiting(pid):
							output = shellAccess.readOutput(pid)
							if output != None and len(output) > 0:
								peer.send("5,"+str(pid)+","+output+"\0")
							if output == None:
								marked.append(pid)
					for pid in marked:
						autoPidWriteBackList.remove(pid)
						peer.send(getStatus(shellAccess,pid,None))		## Let the other side know the system is done
					if len(autoPidWriteBackList) == 0:
						varTimeout = 10
					else:
						varTimeout = 0
					readyList, d1, d2 = select([peer],[],[],varTimeout)
					if len(readyList) > 0:
						commandDataDump = os.read(peer.fileno(),1000)		## use this so not to block
						if len(commandDataDump) == 0:
							break
					else:
						commandDataDump = ''
				peer.close()
				#print 'finished with stay-alive server'
				server.close()
				sys.exit()
			else:
				peer.send("3,-1")
				peerIn = os.fdopen(os.dup(peer.fileno()),'r',1000)
				peerOut = os.fdopen(os.dup(peer.fileno()),'w',1000)
				sheller = ShellProxServer(peerIn, peerOut)
				sheller.shell(shellArgs)
				time.sleep(2)	## finish data transfer
				peerIn.close()
				peerOut.close()
				peer.close()
				#print 'finished with server'
				server.close()
				sys.exit()
		else:
			#print 'appending,',childPid,' to deadThreadList'
			deadThreadList.append((childPid,peer))

	#twcl("--projdir ~seth/project/8260_test1/ --pkg boa --force ")
