
import os, sys, global_vars, threading, thread
from message_handler import *


class Builder:
    def __init__(self, main_win):
        global debug
        
        debug = global_vars.get("debug")
        global_vars.SetBuilder(self)
        self.main_win = main_win
        self.ready = 0
        self.old_pids = []
        self.new_pids = []

    def RegisterLogger(self, log):
        self.log = log
        self.ready = 1

    def SetStateFile(self, projectdir, package_name, prep, config):
        # this will change prep and config settings, for both of them
        # the options are:
        # -1 = don't care
        # 0 = turn off
        # 1 = turn on
        path = os.path.join(projectdir, "build/packagestate", package_name+".st")
        debug(5, "Builder::SetStateFile : path is ",  path)
        prep_str = ""
        config_str = ""

        if os.path.exists(path):
            fd = open(path)
            state_str = fd.read()
            fd.close()
            if state_str.find("bp") != -1:
                prep_str = "bp "
            if state_str.find("cfg") != -1:
                config_str = "cfg "

        if prep == 0:
            prep_str = ""
        if prep == 1:
            prep_str = "bp "
        if config == 0:
            config_str = ""
        if config == 1:
            prep_str = "bp "
            config_str = "cfg "
        try:
            fd = open(path, "w")
            fd.write(prep_str + config_str + "\n")
            fd.close()
        except:
            debug(5, "Builder::SetStateFile : could not write file", path)
 
    def StopBuild(self):
        self.log.StopBuild()
    
    def ForceConfig(self):
        debug(5, "Builder::ForceConfig : being called")
        if not self.ready:
            debug(5, "Builder::ForceConfig : no logger registered!")
            return
        projectdir = global_vars.BuilderPath(self.main_win.GetProjDir())
        package_name = self.main_win.GetPackageName().replace("-", "_")
        if not projectdir or not package_name:
            debug(5, "Builder::TestBuild : missing projectdir or package_name")
            return
        command = "/usr/bin/twcl --projdir %(projectdir)s --pkg %(package_name)s --batch --force configure" % vars()
        status = self.log.ExecuteBuildCommand(command)
        if status == 0:
            self.log.SendHeader("Finished Configure")
        else:
            self.log.SendHeader("Configure Failed")
    

        
    def Configure(self):
        debug(5, "Builder::Configure : being called")
        projectdir = global_vars.BuilderPath(self.main_win.GetProjDir())
        package_name = self.main_win.GetPackageName().replace("-", "_")
        if not projectdir or not package_name:
            debug(5, "Builder::TestBuild : missing projectdir or package_name")
            return
        command = "/usr/bin/twcl --projdir %(projectdir)s --pkg %(package_name)s --force --batch configure" % vars()
        self.log.ExecuteBuildCommand(command)
        self.log.SendHeader("Finished Configure")


    def MakeInstall(self):
        debug(5, "Builder::MakeInstall : being called")
        projectdir = global_vars.BuilderPath(self.main_win.GetProjDir())
        package_name = self.main_win.GetPackageName().replace("-", "_")
        if not projectdir or not package_name:
            debug(5, "Builder::TestBuild : missing projectdir or package_name")
            return
        command = "/usr/bin/twcl --projdir %(projectdir)s --pkg %(package_name)s --batch install" % vars()
        status = self.log.ExecuteBuildCommand(command)
        if status == 0:
            self.log.SendHeader("Finished Install")
        else:
            self.log.SendHeader("Install Failed")

    def ForceCMI(self):
        debug(5, "Builder::ForceCMI : being called")
        debug(5, "Builder::ForceConfig : being called")
        if not self.ready:
            debug(5, "Builder::Make : no logger registered!")
            return
        projectdir = global_vars.BuilderPath(self.main_win.GetProjDir())
        package_name = self.main_win.GetPackageName().replace("-", "_")
        if not projectdir or not package_name:
            debug(5, "Builder::TestBuild : missing projectdir or package_name")
            return
        command = """
        /usr/bin/twcl --projdir %(projectdir)s --pkg %(package_name)s --batch --force install
        """ % vars()
        status = self.log.ExecuteBuildCommand(command)
        if status == 0:
            self.log.SendHeader("Finished Configure, Make, and Install")
        else:
            self.log.SendHeader("Configure, Make, and Install Failed")

    def ForceUCMI(self):
        debug(5, "Builder::ForceUCMI : being called")
        self.ForceUnpack()
        projectdir = global_vars.BuilderPath(self.main_win.GetProjDir())
        package_name = self.main_win.GetPackageName().replace("-", "_")
        if not projectdir or not package_name:
            debug(5, "Builder::TestBuild : missing projectdir or package_name")
            return
        command = "/usr/bin/twcl --projdir %(projectdir)s --pkg %(package_name)s --batch install" % vars()  
        status = self.log.ExecuteBuildCommand(command)
        if status == 0:
            self.log.SendHeader("Finished Unpack, Configure, Make, and Install")
        else:
            self.log.SendHeader("Unpack, Configure, Make, and Install Failed")


    def ForceUnpack(self):
        debug(5, "Builder::ForceUnpack : being called")
        projectdir = global_vars.BuilderPath(self.main_win.GetProjDir())
        package_name = self.main_win.GetPackageName().replace("-", "_")
        builddir = global_vars.BuilderPath(self.main_win.GetBuildDir())
        if not builddir:
            debug(5, "Builder::ForceUnpack : missing builddir")
            return
        
        builddir = builddir.split("/")[0]


        if not projectdir or not package_name:
            debug(5, "Builder::TestBuild : missing projectdir or package_name")
            return
        command = """/usr/bin/twcl --projdir %(projectdir)s --pkg %(package_name)s --batch --nuke prepare --force
        if [ -d %(projectdir)s/src/%(builddir)s.orig ] ; then rm -rf %(projectdir)s/src/%(builddir)s.orig; fi
        mkdir %(projectdir)s/src/%(builddir)s.orig
        cp -ar %(projectdir)s/src/%(builddir)s/* %(projectdir)s/src/%(builddir)s.orig
        """ % vars()
        self.log.ExecuteBuildCommand(command)
        self.log.SendHeader("Finished Unpack")

    def MakeClean(self):
        debug(5, "Builder::MakeClean : being called")
        projectdir = global_vars.BuilderPath(self.main_win.GetProjDir())
        package_name = self.main_win.GetPackageName().replace("-", "_")
        if not projectdir or not package_name:
            debug(5, "Builder::TestBuild : missing projectdir or package_name")
            return
        command = "/usr/bin/twcl --projdir %(projectdir)s --pkg %(package_name)s --batch clean" % vars()
        self.log.ExecuteBuildCommand(command)
        self.log.SendHeader("Finished Clean")


    def Make(self):
        debug(5, "Builder::Make : being called")
        if not self.ready:
            debug(5, "Builder::Make : no logger registered!")
            return
        projectdir = global_vars.BuilderPath(self.main_win.GetProjDir())
        package_name = self.main_win.GetPackageName().replace("-", "_")
        if not projectdir or not package_name:
            debug(5, "Builder::TestBuild : missing projectdir or package_name")
            return

        #if the tar file for this package already exists then executing
        #a make will not work because the builder thinks that it is up to date.
        tar_file_path = projectdir + "/build/tarfiles/" + package_name + ".tar.gz"
        if os.path.exists(global_vars.HostPath(tar_file_path)):
            self.log.ExecuteCommand("rm -f " + global_vars.BuilderPath(tar_file_path) )

        command = "/usr/bin/twcl --projdir %(projectdir)s --pkg %(package_name)s --batch build" % vars()
        self.log.ExecuteBuildCommand(command)
        self.log.SendHeader("Finished Make")

    def ForceCompleteBuild(self):
        if not self.ready:
            debug(5, "Builder::ForceCompleteBuild : no logger registered!")
            return
        projectdir = global_vars.BuilderPath(self.main_win.GetProjDir())
        package_name = self.main_win.GetPackageName().replace("-", "_")
        #builddir = self.main_win.GetBuildDir()
        if not projectdir:
            debug(5, "Builder::TestBuild : missing projectdir")
            return
        if not package_name:
            debug(5, "Builder::TestBuild : missing package_name")
            return

        command = """
/usr/bin/twcl --projdir %(projectdir)s --pkg %(package_name)s --force --batch --nuke
        """ % vars()

        self.log.ExecuteBuildCommand(command)
        self.log.SendHeader("Finished Complete Rebuild")
    
    def GetTarContents(self, tarfilename):
        tarfilename = global_vars.BuilderPath(tarfilename)
        print "tar file is", tarfilename
        extension = tarfilename.split('.')[-1]

        command = ""
        if extension in ["gz", "tgz"]:
            command = ("tar ztf %s" % tarfilename)
        elif extension == "tar":
            command = ("tar tf %s" % tarfilename)
        elif extension == "bz2":
            command = ("bzip2 -dc %s | tar tf -" % tarfilename)
        else:
            debug(5, "Cannot untar file %s because extension is not recognized" % tarfilename)

        return self.log.ExecuteCommand(command, 1)
        
