import sys, os, wizutil, re
from qt import *
from message_handler import *
from widgets import *
import global_vars
HostPath = global_vars.HostPath


# the lbc database will now be changed.  Instead of having just one database, It will
# consist of 4 databases in one.  This way you can check for the existence of entries
# in the built-in, generic, board, and local databases for the lbc.


def diff(a, b):
        "Compares two strings to see if there are differences between them"
        return (a.strip() != b.strip())

#the list of scopes in the order that they should be searched
lbc_scopes = ["local", "board", "generic", "built_in"]

lbc_scope_colors = {}
lbc_scope_colors["local"] = QColor(0, 0, 0)
lbc_scope_colors["board"] = QColor(0, 128, 0)
lbc_scope_colors["generic"] = QColor(0, 0, 200)
lbc_scope_colors["built_in"] = QColor(200, 0, 0)               
lbc_scope_bold = {"local": 1, "board": 0, "generic": 0, "built_in": 0}

# here are the builder defaults
lbc_built_in = {}
lbc_built_in["bld_targ"] = ""
lbc_built_in["spec"] = ""
lbc_built_in["pkg_file"] = ""
lbc_built_in["patches"] = ""
lbc_built_in["bld_dir_name"] = ""
lbc_built_in["cfgopts"] = ""
lbc_built_in["bin"] = ""
lbc_built_in["cflags"] = ""
lbc_built_in["makep"] = ""
lbc_built_in["makec"] = "if [ -s configure ] ; then ./configure ; fi"
lbc_built_in["makerc"] = ""
lbc_built_in["makeb"] = "make"
lbc_built_in["makei"] = ""
lbc_built_in["makest"] = ""
lbc_built_in["makedc"] = "make distclean"

class FieldEntry(QFrame, HandlerChild):
    """
    This is just a compound widget that has a label and an entry field.  Used for
    one line fields in the lbc frame
    """
    field_d = {"pkg_file": "Package File", "patches": "Patches", "bld_dir_name": "Build Directory Name",
        "cflags": "Compile Flags", "cfgopts": "Configuration Options", "makec": "Make Configure",
        "makeb": "Make Build", "makei": "Make Install", "bin": "Binary", "bld_targ": "Build Target",
        "makerc": "Make Re-Configure", "makest": "Make Stage Components", "makedc": "Make Clean",
        "spec": "Spec Filename"}
    fields = ["pkg_file", "patches", "bld_dir_name", "cflags", "cfgopts", "spec", "bin", "bld_targ",
    "makec", "makerc", "makeb", "makei", "makest", "makedc"]
    def __init__(self, parent, name):
        """Set up the frame, a label, and text entry box.  The name corresponds
        to a field in the lbc.  The dictionary mapping of the name is what will be displayed
        in the label"""
        self.parent = parent
        self.name = name
        self.scope = "built_in"

        HandlerChild.__init__(self, "lbc_entry_" + self.name)

        QFrame.__init__(self, parent, name)

        self.label = ColorableLabel(self.field_d[name], self)
        self.line_edit = QLineEdit(self, "line edit for " + name)

        self.vlayout = QVBoxLayout(self)
        self.vlayout.addWidget(self.label, 0)
        self.vlayout.addWidget(self.line_edit, 0)
        QObject.connect(self.line_edit, SIGNAL("textChanged(const QString &)"), self.TextChanged)
        self.text_flag = 0

    def TextChanged(self):
        #we want this to only be called when 
        __doc__ = "this is called when the text changes.it just updates the database value"
        if self.text_flag:
            return
        self.scope = "local"
        self.parent.lbc_database[self.scope][self.name] = str(self.line_edit.text()).strip()
        scope = self.parent.GetFieldScope(self.name)
        self.SetScope(scope)
        self.parent.SetModified(1)
        
        if str(self.line_edit.text()).strip() == "":
            #print "The value for field %s is now empty" % self.name
            #print self.parent.lbc_database
            text = self.parent.GetField(self.name)
            #print "text is" + text
            self.setText(text) 
            self.SetScope(scope)


    def Clear(self):
        self.setText("")
        self.SetScope("built_in")

    def setText(self, text, scope=None):
        """Set the text in the text entry box."""
        self.text_flag = 1
        self.line_edit.setText(text)
        self.text_flag = 0
        if scope:
            self.SetScope(scope)

    def getText(self):
        """Get the text contained in the text entry box."""
        return str(self.line_edit.text())

    def SetScope(self, scope):
        self.scope = scope
        self.label.setColor(lbc_scope_colors[self.scope])
        self.label.setBold(lbc_scope_bold[self.scope])

    def GetScope(self):
        return self.scope

class LBCEditorPopupMenu(QPopupMenu, HandlerChild):
    """This is the popup menu that appears when someone right clicks in the LBCEditor.
    It has the usual cut, copy and paste, but it also shows a sub_menu of available
    build variables and a sub_menu of user defined build_vars from the ecd"""
    builtin_vars = ["$BUILD_ROOT", "$SOURCE_DIR", "$BUILD_DIR", "$ARCH", "$PACKAGE_NAME", "$OPT_FLAGS",
        "$PATH", "$HOST_PREFIX", "$CFG_OPTS", "$DEV_IMAGE", "$CROSS", "$BUILD_TARG", "$BIN_OUT",
        "$PRJ_BASE", "$RPMDIR", "$LINUX_HDR_DIR", "$TARDIR", "$PKG"]
    def __init__(self, parent, custom_buildvars):
        """The bulk of the code is in this constructor.  It fetches the buildvars
        from the ecd and constructs the popup menu"""
        HandlerChild.__init__(self, "lbc_editor_popup_menu", temporary=1)
        QPopupMenu.__init__(self, parent, "LBC Editor Popup Menu")
        self.builtin_vars.sort()
        self.parent = parent
        self.insertItem("&Cut", self.parent.cut)
        self.insertItem("&Copy", self.parent.copy)
        self.insertItem("&Paste", self.parent.paste)
        self.insertSeparator()

        self.built_in_menu = QPopupMenu(self, "LBC Editor built in buildvars Menu")
        QObject.connect(self.built_in_menu, SIGNAL("activated(int)"), self.BuiltInSelection)
        self.buildvar_mapping = {}
        for build_var in self.builtin_vars:
            index = self.built_in_menu.insertItem(build_var)
            self.buildvar_mapping[index] = build_var
        self.insertItem("Build &Variables", self.built_in_menu)

        self.custom_menu = QPopupMenu(self, "LBC Editor custom buildvars Menu")
        QObject.connect(self.custom_menu, SIGNAL("activated(int)"), self.CustomSelection)
        
        for build_var in custom_buildvars:
            index = self.custom_menu.insertItem("$" + build_var)
            self.buildvar_mapping[index] = "$" + build_var
        self.insertItem("Custom &BuildVars", self.custom_menu)

    def BuiltInSelection(self, item_id):
        """This slot is called when the user selects a build variable.  It
        inserts text into the LBCEditor."""
        self.parent.insert(self.buildvar_mapping[item_id])

    def CustomSelection(self, item_id):
        """This slot is called when the user selects a buildvar.  It inserts
        the text into the LBCEditor."""
        self.parent.insert(self.buildvar_mapping[item_id])

class LBCEditor(QMultiLineEdit):
    """This class is just a Multi Line editor with a few modifications achieved
    by overloaded methods.  It handles tabs in the correct way (that is inserting a
    tab and not shifting focus and Pops up an LBCEditorPopupMenu if the right button is
    clicked."""
    def __init__(self, parent, name = "LBCEditor"):
        """ """
        QMultiLineEdit.__init__(self, parent, name)
        self.installEventFilter(self)

    def eventFilter(self, object, event):
        if event.type() == QEvent.KeyPress:
            if event.key() == event.Key_Tab:
                self.insert("\t")
            else:
                self.keyPressEvent(event)
            return 1
        return 0

    def mousePressEvent(self, e):
        if e.button() == e.RightButton:
            self.emit(PYSIGNAL("LeftMouseClicked"), (e.globalPos(),))
        else:
            return QMultiLineEdit.mousePressEvent(self, e)



class AreaEntry(QFrame, HandlerChild):
    __doc__ = """
    This is a frame that contains a radio group of all the fields that need a text area
    and a corresponding text area.  Clicking on a different radio group option updates
    the contents of the text area.  It maintains it's own dictionary (self.database) that
    holds the text for each radio group entry """
    field_d = {"pkg_file": "Package File", "patches": "Patches", "bld_dir_name": "Build Directory Name",
        "cflags": "Compile Flags", "cfgopts": "Configuration Options", "makec": "Make Configure",
        "makeb": "Make Build", "makei": "Make Install", "bin": "Binary", "bld_targ": "Build Target",
        "makerc": "Make Re-Configure", "makest": "Make Stage Components", "makedc": "Make Clean",
        "spec": "Spec Filename"}
    fields = ["pkg_file", "patches", "bld_dir_name", "cflags", "cfgopts", "spec", "bin", "bld_targ",
    "makec", "makerc", "makeb", "makei", "makest", "makedc"]
    def __init__(self, parent, lbc_database, name = "AreaFrame"):
        self.parent = parent
        self.area_fields = parent.area_fields
        self.scope = "built_in"
        self.text_flag = 0
        QFrame.__init__(self, parent, name)
        HandlerChild.__init__(self, "lbc_area_frame")

        self.grid_layout = QGridLayout(self, 2, 2, 10)
        self.lbc_database = lbc_database


        #set up the radio group, field_d is a mapping of field name to field description
        self.radio_group = QVButtonGroup("Fields", self, name)
        self.vradio_layout = QVBoxLayout()
        self.radio_buttons = {}
        for item in self.area_fields:
            self.radio_buttons[item] = ColorableRadioButton(self.radio_group, item)
            self.radio_buttons[item].setText(self.field_d[item])
            self.radio_buttons[item].setMinimumSize(self.radio_buttons[item].sizeHint())
            self.vradio_layout.addWidget(self.radio_buttons[item])

        #now set the first radio group and add this group to the grid layout
        self.radio_buttons[self.area_fields[0]].setOn(1)
        self.radio_group.setMinimumSize(self.radio_group.sizeHint())
        self.grid_layout.addMultiCellWidget(self.radio_group, 0, 1, 0, 0)

        #add the text area and label to the widget
        self.text_area = LBCEditor(self, "area entry box")
        QObject.connect(self.text_area, PYSIGNAL("LeftMouseClicked"), self.EditorLeftMouseClickedSlot)
        self.grid_layout.addWidget(self.text_area, 1, 1)
        self.label = ColorableLabel("Field Title", self)
        self.grid_layout.addWidget(self.label, 0, 1)

        #connect signals to methods, set the current field and act like it was just
        #clicked
        QObject.connect(self.radio_group, SIGNAL("clicked(int)"), self.ClickedSlot)
        QObject.connect(self.text_area, SIGNAL("textChanged()"), self.TextChanged)
        self.current_field = self.area_fields[0]
        self.ClickedSlot(0)

    def EditorLeftMouseClickedSlot(self, pos):
        #build_vars = self.parent.GetBuildVars()
        editor_popup_menu = LBCEditorPopupMenu(self.text_area, self.parent.GetBuildVars())
        editor_popup_menu.popup(pos)

    def TextChanged(self):
        #we want this to only be called when 
        __doc__ = "this is called when the text changes.it just updates the database value"
        if self.text_flag:
            return
        key = self.current_field
        self.scope = "local"
        self.lbc_database[self.scope][key] = str(self.text_area.text())
        self.SetScope(self.parent.GetFieldScope(key))

        self.parent.SetModified(1)
        
        if str(self.text_area.text()) == "":
            text = self.parent.GetField(key)
            self.SetText(text)

    def Clear(self):
        self.SetText("")
        for key in self.radio_buttons.keys():
            self.radio_buttons[key].setBold(lbc_scope_bold["built_in"])
            self.radio_buttons[key].setColor(lbc_scope_colors["built_in"])
            self.label.setBold(lbc_scope_bold["built_in"])
            self.label.setColor(lbc_scope_colors["built_in"])


    def SetScope(self, scope):
        self.scope = scope
        key = self.current_field
        self.radio_buttons[key].setBold(lbc_scope_bold[self.scope])
        self.radio_buttons[key].setColor(lbc_scope_colors[self.scope])
        self.label.setBold(lbc_scope_bold[self.scope])
        self.label.setColor(lbc_scope_colors[self.scope])

    def UpdateContents(self):
        self.SetText(self.parent.GetField(self.current_field))
        self.SetScope(self.parent.GetFieldScope(self.current_field))

    def ClickedSlot(self, id):
        __doc__ = """
        This is called when a user clicks on a button in the radio group.  It writes
        the value of the old field into the database, then figures out which button
        was selected and updates the text area for the new selected field"""
        self.lbc_database[self.scope][self.current_field] = str(self.text_area.text())
        selected_field = ""
        for field in self.area_fields:
            if self.radio_buttons[field].isOn():
                selected_field = field
                break

        if selected_field == "": print "Error in AreaEntry::ClickedSlot"
        self.current_field = selected_field
        scope = self.parent.GetFieldScope(selected_field)
        self.SetText(self.parent.GetField(selected_field))
        self.label.setText(self.field_d[selected_field])
        self.SetScope(scope)

    def SetText(self, text):
        self.text_flag = 1
        self.text_area.setText(text)
        self.text_flag = 0

    def NewLBC(self):
        text = self.parent.GetField(self.current_field)
        self.SetText(text)
        self.scope = self.parent.GetFieldScope(self.current_field)
        for field in self.area_fields:
            scope = self.parent.GetFieldScope(field)
            self.radio_buttons[field].setBold(lbc_scope_bold[scope])
            self.radio_buttons[field].setColor(lbc_scope_colors[scope])
            
class LBCFrame(QFrame, HandlerChild):
    line_fields = ["pkg_file", "bld_dir_name", "cflags", "cfgopts"]
    area_fields = ["patches", "spec", "bin", "bld_targ", "makec", "makerc", "makeb", "makei", "makest", "makedc"]
    fields = ["pkg_file", "patches", "bld_dir_name", "cflags", "cfgopts", "spec", "bin", "bld_targ",
    "makec", "makerc", "makeb", "makei", "makest", "makedc"]
    field_d = {"pkg_file": "Package File", "patches": "Patches", "bld_dir_name": "Build Directory Name",
        "cflags": "Compile Flags", "cfgopts": "Configuration Options", "makec": "Make Configure",
        "makeb": "Make Build", "makei": "Make Install", "bin": "Binary", "bld_targ": "Build Target",
        "makerc": "Make Re-Configure", "makest": "Make Stage Components", "makedc": "Make Clean",
        "spec": "Spec Filename"}
    
    def __init__(self, parent, main):
        global debug, log
        debug = sys.modules["__main__"].debug
        log = global_vars.get("log")
       
        self.main = main
        #initialize the lbc_database
        self.lbc_database = {}
        for scope in lbc_scopes:
            self.lbc_database[scope] = {}
        self.lbc_database["built_in"] = lbc_built_in
        
        #print "lbc database is", self.lbc_database
        QFrame.__init__(self, parent, "LBC Frame")
        HandlerChild.__init__(self, "lbc_frame")
        
        #set modified to 0. This will tell us if the object has any unsaved data
        self.modified = 0
        
        #set up the widget layouts
        self.hlayout = QHBoxLayout(self)
        self.hlayout.addSpacing(10)
        self.vlayout = QVBoxLayout(self.hlayout)
        self.vlayout2 = QVBoxLayout(self.vlayout)
        self.hlayout.addSpacing(10)

        #create the 4 color codes
        self.color_groupbox = QGroupBox(2, Qt.Horizontal, "Color Codes", self)
        
        self.color_code_labels = {}
        self.color_code_labels["built_in"] = ColorableLabel("Built In", self.color_groupbox)
        self.color_code_labels["generic"] = ColorableLabel("Generic", self.color_groupbox)
        self.color_code_labels["board"] = ColorableLabel("Board", self.color_groupbox)
        self.color_code_labels["local"] = ColorableLabel("Local", self.color_groupbox)

        for scope in lbc_scopes:
            self.color_code_labels[scope].setColor(lbc_scope_colors[scope])
            self.color_code_labels[scope].setBold(lbc_scope_bold[scope])
            #self.vlayout2.addWidget(self.color_code_labels[scope])
        self.vlayout2.addWidget(self.color_groupbox)

        #create the 4 line entries for the one line fields
        self.line_entries = {}
        for field in self.line_fields:
            self.line_entries[field] = FieldEntry(self, field)

        self.field_grid_layout = QGridLayout(2, 2, 10, "grid layout")
        self.field_grid_layout.addWidget(self.line_entries["pkg_file"], 0, 0)
        self.field_grid_layout.addWidget(self.line_entries["bld_dir_name"], 0, 1)
        self.field_grid_layout.addWidget(self.line_entries["cflags"], 1, 0)
        self.field_grid_layout.addWidget(self.line_entries["cfgopts"], 1, 1)
        self.vlayout.addLayout(self.field_grid_layout, 0)

        #now instantiate and add the area_entry compound widget
        self.area_entry = AreaEntry(self, self.lbc_database)
        self.vlayout.addWidget(self.area_entry, 1)
        
        self.vlayout.addSpacing(10)
        self.hlayout.activate()

        self.SetListener("GetLBCField", self.GetField)
        self.SetListener("GetLBCFieldScope", self.GetFieldScope)
        self.SetListener("SetLBCField", self.SetField)
        self.SetListener("ReadLBCFile", self.GetLBC)
        self.SetListener("WriteLBCFile", self.WriteLBC)
        self.SetListener("WriteFullLBCFile", self.WriteFullLBC)
        self.SetListener("MakeMinimalLBC", self.MakeMinimalLBC)
        self.modified = 0

    def Close(self):
        for key in self.line_entries.keys():
            self.line_entries[key].Clear()
        self.area_entry.Clear()
        self.lbc_database["local"].clear()
        self.lbc_database["board"].clear()
        self.lbc_database["generic"].clear()
        self.SetModified(0)
        self.setEnabled(0)

    def GetBuildVars(self):
        return self.main.GetBuildVars()
    
    def MakeMinimalLBC(self, pkg_file, lbc_path):
        print "Make Minimal LBC, package file is"
        self.lbc_database["local"].clear()
        self.lbc_database["generic"].clear()
        self.lbc_database["board"].clear()
        self.SetField("local", "pkg_file", os.path.basename(pkg_file))
        build_dir = self.DetermineBuildDirectory()
        self.SetField("local", "makec", "if [ -s configure ] ; then ./configure ; fi")
        self.SetField("local", "makeb", "make")
        self.SetField("local", "makei", "make install prefix=$BUILD_ROOT")
        self.SetField("local", "makedc", "make clean")
        if build_dir:
            self.SetField("local", "bld_dir_name", build_dir)
        self.WriteLBC(lbc_path)
 

    def UpdateLBCDatabase(self):
        for field in self.line_fields:
            scope = self.line_entries[field].GetScope()
            self.lbc_database[scope][field] = self.line_entries[field].getText()

    def ViewConfigureHelp(self):
        project_dir = self.EmitMessage("GetProjectDirectory")
        build_dir = self.GetField("bld_dir_name")
        command = "cd %s; ./configure --help" % os.path.join(project_dir, "src", build_dir)
        (status, text) = global_vars.log.ExecuteCommand(command, 1)
        if status == 1:
            QMessageBox.warning(self, "Error", """<qt>There was an error when running configure --help
            for this project.  The output of the command was:<br><pre>%s</pre>""" % text)
        else:
            usage = ""
            for line in text.split("\n"):
                if len(line) and line[0] == "+": 
                    continue
                usage += line + "\n"
            box = OutputDialog(self, "configure --help output", "Configure Information")
            box.SetText(usage)
            box.show()

    def DetermineBuildDirectory(self, filename=None):
        #if we didn't get a filename
        #we do this test because it's possible that this was called from a menu item and
        #in that case it would pass the menuid as the first parameter.  So we test to see if
        #we got a string or not.
        if type(filename) != type("string"):
            filename = None
            projectdir = self.EmitMessage("GetProjectDirectory")
            pkg_path = os.path.join(projectdir, "Packages", "%s", self.GetField("pkg_file"))
            for scope in lbc_scopes:
                #print "Checking for file : ", pkg_path % scope
                if os.path.exists(global_vars.HostPath(pkg_path % scope)):
                    filename = pkg_path    % scope
                    break
        #print "filename is", filename
        if filename == None:
            QMessageBox.warning(self, "Error", """<qt>There was an error when determining the build directory
            for the current package.  Please see the log window for more information</qt>""")
            return ""

        if filename.endswith("tgz") or filename.endswith("tar.gz") or filename.endswith("tar"):
            (status, output) = global_vars.builder.GetTarContents(filename)
            if status != 0:
                QMessageBox.warning(self, "Error", """<qt>There was an error when determining the build directory
            for the current package.  Please see the log window for more information</qt>""")
                return ""
            for line in output.split("\n"):
                if len(line) == 0 or line[0] == "+":
                    continue
                build_dir = os.path.dirname(line)
                self.SetField("local", "bld_dir_name", build_dir)
                return build_dir            
        matches = re.findall(r".*/([^/]*)-", filename)
        print "name matches are", matches
        if len(matches):
            self.SetField("local", "bld_dir_name", matches[0])
            return matches[0]
        return ""

    
    def GetField(self, name, update = 1):
        if update:
            self.UpdateLBCDatabase()
        #run through each of the scopes looking for then entry
        #if none is found then just return ""
        for scope in lbc_scopes:
                if self.lbc_database[scope].has_key(name) and self.lbc_database[scope][name].strip():
                    return self.lbc_database[scope][name]
        return ""

    def GetFieldScope(self, name):
        for scope in lbc_scopes:
                if self.lbc_database[scope].has_key(name) and self.lbc_database[scope][name].strip():
                    return scope
        return "built_in"

    def SetField(self, scope, field, value):
        #print self.lbc_database
        if not field in self.fields:
            debug(5, "LBCFrame::SetField : field %s is not valid" % field)
            return
        self.lbc_database[scope][field] = str(value)
        if field in self.line_fields:
            self.line_entries[field].setText(self.lbc_database[scope][field])
            self.line_entries[field].SetScope(scope)
        if field in self.area_fields:
            self.area_entry.UpdateContents()

    def GetLBC(self, projectdir, lbc_name):
        __doc__ = """
        this clears out the database and then reads the lbc file from each of the
        three lbc directories, overwriting as it goes down them"""

        if not projectdir or not lbc_name:
            debug(5, "LBCFrame::GetLBC : projectdir or lbc_name missing")
            return

        #first clear out the old database values, then read them in from config file
        for scope in lbc_scopes:
            if scope != "built_in":
                self.lbc_database[scope].clear()

        path = os.path.join(projectdir, "config-data/buildcontrol/") + "%s/" + lbc_name + ".lbc"

        debug(5, "lbc path is", path)

        #print "lbc_database", self.lbc_database, "\n\n"

        if os.path.exists(path % "generic"):
            conf_d = wizutil.parse_config(path % "generic", keep_comments = 1)
            self.lbc_database["generic"].update(conf_d)
        if os.path.exists(path % "board"):
            conf_d = wizutil.parse_config(path % "board", keep_comments = 1)
            self.lbc_database["board"].update(conf_d)
        if os.path.exists(path % "local"):
            conf_d = wizutil.parse_config(path % "local", keep_comments = 1)
            self.lbc_database["local"].update(conf_d)

        self.lbc_database["built_in"] = lbc_built_in

        for scope in lbc_scopes:
            for key in self.lbc_database[scope].keys():
                self.lbc_database[scope][key] = self.lbc_database[scope][key].strip()
                
        #update the database that the area_entry maintains and each line field
        for field in self.line_fields:
            self.line_entries[field].setText(self.GetField(field, 0))
            self.line_entries[field].SetScope(self.GetFieldScope(field))
        
        self.area_entry.NewLBC()

        if self.GetField("bld_dir_name"):
            self.main.SetBuildDir(self.GetField("bld_dir_name"))
        self.setEnabled(1)

        #print "lbc_database is", self.lbc_database, "\n\n"
        self.SetModified(0)
        
    def WriteFullLBC(self, path):
        self.UpdateLBCDatabase()
        try:
            f = open(HostPath(path), "w")
        except:
            debug(5, "LBCFrame::WriteFullLBC : couldn't open %s for writing" % path)
            return 0

        #write each entry back to the file
        for key in self.lbc_database["local"].keys():
            f.write('%'+key+'\n')
            f.write(self.lbc_database["local"][key].strip()+'\n\n')
        f.close()
        self.SetModified(0)
        
    def WriteLBC(self, filename):
        __doc__ = "This figures out what fields to write to the local directory and then writes them"

        debug(5, "LBCFrame::WriteLBC : being called")

        projectdir = self.EmitMessage("GetProjectDirectory")
        lbc_name = self.EmitMessage("GetPackageName")

        if not projectdir or not lbc_name:
            debug(5, "LBCFrame::WriteLBC : projectdir or lbc_name missing")
            return

        for field in self.line_fields:
            self.lbc_database[field] = self.line_entries[field].getText()
        keys = []
        #if a field was totally blank then don't write it!
        for field in self.fields:
            if self.lbc_database["local"].has_key(field) and self.lbc_database["local"][field].strip() != "": keys.append(field)
    
        path = os.path.join(projectdir, "config-data/buildcontrol/local", lbc_name + ".lbc")
        path = HostPath(path)
        try:
            f = open(path, "w")
        except:
            debug(5, "LBCFrame::WriteLBC : couldn't open %s for writing" % path)
            return 0

        #write each entry back to the file
        for key in keys:
            f.write('%'+key+'\n')
            f.write(self.lbc_database["local"][key].strip()+'\n\n')
        f.close()
        self.SetModified(0)
         
    #this is called when edits are made or when a file is opened or saved
    #to indicate if the lbc itself has been modified or not.  It modifies the tab title
    #to reflect if it has been modified or not
    def SetModified(self, modified):
        if self.modified == modified:
            return
        self.modified = modified
        self.EmitMessage("SetTabModified", ("LBC File", modified))

    def GetModified(self):
        return self.modified
