#this is an attempt at having an observer, or one main handler
import os, sys, re
from qt import *

import global_vars


class Message:
    """"This class encapsulates all parts of a message it is used by the Handler and
    HandlerChild classes"""
    sender = None
    receiver = None
    function = None
    message = None
    arguments = None
    def __init__(self, sender, receiver, message, function):
        "This basic constructor creates the Message object"
        self.sender = sender
        self.receiver = receiver
        self.message = message
        self.function = function

class Handler(QObject):
    """This is a singleton class that acts as the observer/intermediary for Messages between
    objects that inherit for HandlerChild"""
    def __init__(self):
        "constructor takes no arguments, just initializes a few values"
        global debug
        debug = global_vars.get("debug")
        self.objects = {}
        self.message_listeners = {}

    def Register(self, object, name, temporary = 0):
        """Called by the constructor of each HandlerChild object, to register,
        an object must pass itself and a name.  temporary is set if the object will replace
        another object with the same name"""
        if temporary:
            self.objects[name] = object
            return name

        if self.objects.has_key(name):
            return None
        else:
            self.objects[name] = object
            return name

    def Unregister(self, name):
        """Unregister an object with the Handler.  This essentially disconnects
        all listeners that the object with this name has set up"""
        if self.objects.has_key(name):
            #del self.objects[name]
            for key in self.message_listeners.keys():
                for i in range(len(self.message_listeners[key])):
                    if i > len(self.message_listeners[key]):
                        break
                    while i > len(self.message_listeners[key]) and self.message_listeners[key][i].receiver == name:
                        del self.message_listeners[key][i]

    def Debug(self):
        """Send a list of all registered objects to the debug method"""
        keys = self.objects.keys()
        keys.sort()
        for key in keys:
            debug(5, "Object: %s => %s" % (key, str(self.objects[key])))

    def SetListener(self, receiver, message, function):
        """Register a listener for an object.  This will receive a specific message regardless
        of who sends it.
            - receiver = name of the object that will receive the message
            - message = a message (in the form of a string) that it will listen for
            - function = the method that will be called when the message is sent, (e.g. self.foo)
        """

        if not self.message_listeners.has_key(message):
            self.message_listeners[message] = []
        self.message_listeners[message].append(Message("all", receiver, message, function))


    def SetObjectListener(self, sender, receiver, message, function):
        """Register a listener for an object.  This will receive a specific message from a specific
        object.
            - sender = name of the object that will send the message
            - receiver = name of the object that will receive the message
            - message = a message (in the form of a string) that it will listen for
            - function = the method that will be called when the message is sent, (e.g. self.foo)
        """

        if not self.message_listeners.has_key(message):
            self.message_listeners[message] = []
        self.message_listeners.append(Message(sender, receiver, message, function))

    def SendMessage(self, sender_name, receiver_name, message, arglist):
        """Send a message to an object.  If the object has no listener set up for this
        message then nothing will happen.
            - sender_name = name of object that is sending the message
            - receiver_name = name of object that the message is being sent to
            - message = the message in the form of a string
            - a sequence (tuple or list) of arguments that will be handed to the receiving function
        This will call the function set up by the listener with the arguments in arglist.  If that function
        returns any values, those will be returned from this function (thus allowing a messages to be sent to
        "gather" information)
        """
        debug(5, "SendMessage:: from %s, to %s, message %s, args %s" % (sender_name, receiver_name, message, str(arglist)))
        if self.objects.has_key(sender_name):
            sender = self.objects[sender_name]
        else:
            debug(5, "SendMessage::%s is not registered" % sender_name)
            return None
        if not self.message_listeners.has_key(message):
            debug(5, "SendMessage::%s is not a registered message" % message)
            return
        if not self.objects.has_key(receiver_name):
            debug(5, "SendMessage::%s is not in the object list" % receiver_name)
        for message_listener in self.message_listeners[message]:
            if message_listener.receiver == receiver_name and message_listener.message.strip() == message.strip():
                return self.RunFunction(message_listener.function, arglist)

    def EmitMessage(self, sender_name, message, arglist):
        """Broadcast a message to all objects registered.  Any objects listening for this message
        will have their registered listener methods called.  It will return a list of all of the
        values returned by each listener.
            - sender_name = name of sender object
            - message = message in the form of a string
            - arglist = list of arguments to be passed to the registered listener methods
         """

        global_vars.d["debug"](5, "EmitMessage:: from %s, message %s, args %s" % (sender_name, message, str(arglist)))
        if self.objects.has_key(sender_name):
            sender = self.objects[sender_name]
        else:
            debug(5, "EmitMessage::%s is not registered" % sender_name)
            return None
        if not self.message_listeners.has_key(message):
            debug(5, "EmitMessage::%s is not a registered message" % message)
            return
        return_list = []
        for message_listener in self.message_listeners[message]:
            if message_listener.message.strip() == message.strip() and self.objects.has_key(message_listener.receiver):
                return_list.append(self.RunFunction(message_listener.function, arglist))
                debug(5, "argslist is %s, returnlist is %s" % (arglist, return_list))
        if len(return_list) == 1:
            return return_list[0]
        else:
            return return_list


    def GetObject(self, object_name):
        """Return the object that is registered with the handler with this name.
        Useful if you need the handle to an object and passing references becomes
        cumbersome."""
        debug(5, "GetObject:: %s is requested" % object_name)
        if self.objects.has_key(object_name):
            object = self.objects[object_name]
        else:
            debug(5, "GetObject::%s is not registered" % object_name)
            return None
        return object


    def RunFunction(self, func, arglist):
        """Call a registered listener function with a specified argument list.  If there are more
        arguments in the arglist than the listener function expects, the excess arguments are trimmed
        off.  If there are too few, an exception is raised.  The registered functions may be normal
        functions and need not be tailored to be called."""
        arity = 0
        try:
            value = apply(func, arglist)
            return value
        except TypeError, e:
            if str(e).find("too many arguments") == -1:
                raise TypeError, e
            arity = int(re.findall("expected [0-9]", str(e))[0][-1])
            try:
                return apply(func, arglist[0:arity]) #if this gets an exception, then there weren't enough arguments passed with message
            except TypeError, e:
                                print str(e)
                                if str(e).find("too many arguments") == -1:
                                        raise TypeError, e
                                return apply(func, arglist[0:arity-1])


class HandlerChild(QObject):
    """This class is essentially a python mix-in.  Objects that want to be able to use the
    Handler object need to inherit from HandlerChild.  Objects that inherit from this should
    not call Handler methods directly.  Instead, they should call call these methods"""
    def __init__(self, name, temporary = 0):
        """Unfortunately, this constructor must be called explicitly from the descendent objects.
        it should be passed a name that can be nearly gauranteed to be unique.  If it ends up
        not being unique then the constructor will modify the name to be unique.  If temporary
        is set to 1, then the object will over-write any other object with the same name if one
        exists with the Handler object."""
        self._name = name
        #self._handler = global_vars.d["handler"]
        self._handler = global_vars.get("handler")
        i = 0
        if self._handler.Register(self, self._name, temporary) == None:
            while self._handler.Register(self, self._name + str(i)) == None:
                i = i + 1
                self._name = self._name + str(i)

    def EmitMessage(self, message, arglist = []):
        """Broadcast a global message through the Handler object.  Essentially is a
        thin wrapper for Handler::EmitMessage.
            - message = a message in the form of a string to be broadcast
            - arglist = a sequence of arguments that will be applied to listener functions (optional)
                """
        value = self._handler.EmitMessage(self._name, message, arglist)
        return value

    def SendMessage(self, receiver_name, message, arglist = []):
        """Send a a message to a specific object.  Essentially is a thin wrapper for
        Handler::SendMessage.
            - receiver_name = name of the object that the message is being sent to.
            - message = a message in the form of a string to be broadcast
            - arglist = a sequence of arguments that will be applied to listener functions (optional)
        """

        value = self._handler.SendMessage(self._name, receiver_name, message, arglist)
        return value

    def SetListener(self, message, function):
        """Register a listener for a certain message.  When any object broadcasts this message
        or sends a messsage to this object, the function given will be invoked with the arguments
        that were sent with the message
            - message = the message that the listener is listening for in the form of a string
            - function = method that will be called when the message is received
        """
        self._handler.SetListener(self._name, message, function)

    def SetObjectListener(self, object, message, function):
        """Register a listener for a certain message from a specific object.
        When the specified object broadcasts this message
        or sends a messsage to this object, the function given will be invoked with the arguments
        that were sent with the message
            - object = name of the object that is being listened for
            - message = the message that the listener is listening for in the form of a string
            - function = method that will be called when the message is received
        """

        self._handler.SetObjectListener(object_name, self.__name, message, function)

    def GetHandlerName(self):
        """Return the name of this object that was registered with the Handler"""
        return self._name

    def Unregister(self):
        """Unregister this object with the Handler.  Just a wrapper to Handler::Unregister"""
        self._handler.Unregister(self._name)




