from socket import *
from select import *
import time
import os, sys, thread, string

REMOTE_CONNECTED = 0

def timedRead(sock, timer=0, buffer=1000, default=None):
    readyList, d1, d2 = select([sock],[],[],timer)
    if len(readyList) > 0:
        #return os.read(sock.fileno(),buffer)
        return sock.recv(1000)
    else:
        return default

class ShellAccess:
    __verbHandler = []
    def runCommandAck(self, pid, data):
        #print 'in runCommandAck',pid, data
        try:
            if int(data) == -2:
                self.commandDict[pid].setCommandPid(0)
                return (pid, data)
            self.commandDict[int(data)]=self.commandDict[pid]
            self.commandDict[pid].setCommandPid(int(data))
            del self.commandDict[pid]
            return (pid, data)
        except Exception, e:
            try:
                self.commandDict[pid].setCommandPid(-3,e)
            except:
                pass
            #print 'returning',e
            return (-3, e)
    __verbHandler.append(runCommandAck)          ## Verb 0

    def killAck(self, pid, data):
        try:
            self.commandDict[pid].setKillStatus(int(data))
            return (pid, data)
        except Exception, e:
            try:
                self.commandDict[pid].setKillStatus(-3,e)
            except:
                pass
            return (-3, e)
    __verbHandler.append(killAck)

    def getStatusAck(self, pid, data):
        try:
            self.commandDict[pid].setCommandStatus(int(data))
            return (pid, data)
        except Exception, e:
            try:
                self.commandDict[pid].setCommandStatus(-3,e)
            except:
                pass
            return (-3, e)
    __verbHandler.append(getStatusAck)             ## Verb 2

    def writeInputAck(self, pid, data):
        try:
            self.commandDict[pid].setInputReturnStatus(int(data))
            return (pid, data)
        except Exception, e:
            try:
                self.commandDict[pid].setInputReturnStatus(-3,e)
            except:
                pass
            return (-3, e)
    __verbHandler.append(writeInputAck)             ## Verb 3

    def outputWaitingAck(self, pid, data):
        try:
            self.commandDict[pid].setOutputWaiting(int(data))
            return (pid, data)
        except Exception, e:
            try:
                self.commandDict[pid].setOutputWaiting(-3,e)
            except:
                pass
            return (-3, e)
    __verbHandler.append(outputWaitingAck)          ## Verb  4

    def readOutputAck(self, pid, data):
        try:
            if data == "\1\2\2\1":
                data = 'None'
            self.commandDict[pid].setOutput(data)
            return (pid, data)
        except Exception, e:
            try:
                self.commandDict[pid].setOutput(str(e),e)
            except:
                pass
            return (-3, e)
    __verbHandler.append(readOutputAck)             ## Verb 5

    def getLastErrorAck(self, pid, data):
        try:
            self.lastRemoteError = Exception(data)
            return (pid, data)
        except Exception, e:
            self.lastRemoteError = Exception("Internal Error:"+str(e))
            return (-3, e)
    __verbHandler.append(getLastErrorAck)           ## Verb 6

    def getNewErrorAck(self, pid, data):
        try:
            self.remoteHasNewError = int(data)
            return (pid, data)
        except Exception, e:
            self.remoteHasNewError = -3
            return (-3, e)
    __verbHandler.append(getNewErrorAck)            ## Verb 7

    def getCommandsAck(self, pid, data):
        try:
            commandListOld = string.split(data,"\2")
            currentCommands = []
            for command in commandList:
                tempList = string.split(command,"\1")
                currentCommands.append((int(tempList[0]),tempList[1]))
            return (pid, data)
        except Exception, e:
            currentCommands = ((-3,e))
            return (-3, e)
    __verbHandler.append(getCommandsAck)            ## Verb 8

    def setAutoWriteBackAck(self, pid, data):
        try:
            self.commandDict[pid].setAutoWriteBackStatus(int(data))
            return (pid, data)
        except Exception, e:
            self.commandDict[pid].setAutoWriteBackStatus(-3,e)
            return (-3, e)
    __verbHandler.append(setAutoWriteBackAck)       ## Verb 9

    def clearFinishedCommandsAck(self, pid, data):
        return (pid, data)
    __verbHandler.append(clearFinishedCommandsAck)      ## Verb 10

    def __init__(self):
        self.commandDict = {}
        self.threadRunning = 0

    def __del__(self):
        self.__shutdown()

    def connect(self, hostArg = None, portArg = None, timeOut = 10):
        global REMOTE_CONNECTED
        host = os.getenv('SHELLPROX_HOST')
        if host == None and hostArg == None:
            raise Exception("Host not set! Please set environment variable 'SHELLPROX_HOST' to remote shell host!")
        try:
            portStr = os.getenv('SHELLPROX_PORT')
            port = int(portStr)
        except:
            port = 49999    ## Default
        if hostArg != None:
            host = hostArg
        if portArg != None:
            port = portArg
        self.channel = socket(AF_INET,SOCK_STREAM)
        self.channel.connect((host,port))
        self.channel.send("\1,STAY_ALIVE_SERVER\0")
        firstAck = timedRead(self.channel, timeOut)
        if firstAck != "\1,ACK\0":
            self.channel.close()
            REMOTE_CONNECTED = 0
        else:
            thread.start_new_thread(self.threadReader,())
            REMOTE_CONNECTED = 1
        return REMOTE_CONNECTED

    def __shutdown(self):
        global REMOTE_CONNECTED
        REMOTE_CONNECTED = 0
        timer = 0
        try:
            while self.threadRunning == 1 and timer < 10:
                time.sleep(1)
                timer = timer + 1
            self.channel.send("\1,SHUT_DOWN_SOCKET\0")
            lastRead = timedRead(self.channel,2)
            self.handleVerbs(lastRead)
            self.channel.close()
        except AttributeError:
            pass
        return 1

    def handleVerbs(self, dataDump):
        try:
            commandList = string.split(dataDump, "\0")
        except:
            #print 'dataDump split failed:',dataDump
            return
        for command in commandList:
            if len(command) == 0:
                continue
            try:
                #print 'processing:',command
                index = string.index(command,",")
                type = int(command[:index])
                command = command[index+1:]
                index = string.index(command,",")
                pid = int(command[:index])
                data = command[index+1:]
                try:
                    if int(data) == -2:
                        self.lastRemoteError = None
                        self.channel.send("6,0,0")
                except:
                    pass
                self.__verbHandler[type](self,pid, data)
            except Exception, e:
                pass
                #print 'READ ERROR!!!!! (handleVerbs)',e

    def threadReader(self):
        global REMOTE_CONNECTED
        self.threadRunning = 1
        while REMOTE_CONNECTED:
            dataDump = timedRead(self.channel, 10)
            if dataDump == None or len(dataDump) == 0:
                continue
            while dataDump[-1] != '\0':
                dataDump = dataDump + timedRead(self.channel)
            self.handleVerbs(dataDump)
        self.threadRunning = 0

    def registerCommand(self, commandObject):
        keyList = self.commandDict.keys()
        maxKey = -4
        set = 0
        for key in keyList:
            if key < 0:
                if self.commandDict[key] == None:
                    self.commandDict[key] = commandObject
                    commandObject.setCommandPid(key)
                    set = 1
                else:
                    maxKey = key
        if not set:
            self.commandDict[(maxKey - 1)] = commandObject
            commandObject.setCommandPid(maxKey-1)

    def send(self, commandObject, type, payload):
        cOpid = commandObject.getPid()
        if self.commandDict[cOpid] != commandObject:
            return (-3, Exception("Pid Mismatch"))
        else:
            try:
                sent = self.channel.send(str(type)+","+str(cOpid)+","+str(payload)+"\0")
                return (sent,"Data sent")
            except Exception, e:
                return (-3, e)
MainShellAccessProxy = ShellAccess()
            
class Command:
    def __inputReaderThread(self):
        global REMOTE_CONNECTED
        self.threadRunning = 1
        #print 'command.inputReaderThread: thread running'
        while REMOTE_CONNECTED and self.commandStatus < 0:
            dataRead = os.read(self.in_fd,1000)
            #print 'got ',dataRead,'from input'
            if len(dataRead) == 0:
                break
            self.proxAccess.send(self, 3, dataRead)
        #print 'command.inputReaderThread: thread stopped'
        self.threadRunning = 0

    def __init__(self, command, inputSrc = None, outputDst = None):
        global MainShellAccessProxy, REMOTE_CONNECTED
        self.proxAccess = MainShellAccessProxy
        self.myPid = -4
        self.killStatus = 0
        self.commandStatus = -2
        self.inputReturnStatus = -2
        self.remoteOutputWaiting = 0
        self.recentRead = 0
        self.autoWriteBackStatus = 0
        self.outputList = []
        if type(inputSrc) == type(0):
            self.in_fd = inputSrc
        elif inputSrc == None:
            self.in_fd = None
        else:
            self.in_fd = inputSrc.fileno()
        if type(outputDst) == type(0):
            self.out_fd = outputDst
        elif outputDst == None:
            self.out_fd = None
        else:
            self.out_fd = outputDst.fileno()
        if not REMOTE_CONNECTED:
            self.proxAccess.connect()
        if not REMOTE_CONNECTED:
            raise Exception("No Connection After 10 seconds!")
        self.proxAccess.registerCommand(self)
        oldPid = self.myPid
        print 'temp pid = ', self.myPid
        self.proxAccess.send(self,0,command)
        timeout = 0
        while self.myPid == oldPid and timeout < 100:
            #print 'self.myPid =',self.myPid
            timeout = timeout + 1
            time.sleep(.1)
        print 'new pid = ', self.myPid

        if self.myPid == oldPid:
            raise Exception("Connection Time Out")
        if self.myPid == 0:
            timeout = 0
            while self.proxAccess.lastRemoteError == None and timeout < 100:
                timeout = timeout + 1
                time.sleep(.1)
            if self.proxAccess.lastRemoteError != None:
                raise Exception(self.proxAccess.lastRemoteError)
            else:
                raise Exception("Remote Error! (unknown)")
        if outputDst != None:
            self.proxAccess.send(self,9,None)
            timeout = 0
            while self.autoWriteBackStatus == 0 and timeout < 100:
                timeout = timeout + 1
                time.sleep(.1)
            if self.autoWriteBackStatus == 0:
                raise Exception ("Connection Time Out")
        if self.in_fd != None:
            thread.start_new_thread(self.__inputReaderThread,())

    def __errorFunc(self, numeric, passedException, default="Default Error"):
        if numeric == -3:
            if passedException != None:
                raise passedException
            else:
                raise Exception(default)
        elif numeric == -2:
            if self.proxAccess.lastRemoteError != None:
                raise Exception(self.proxAccess.lastRemoteError)
            else:
                raise Exception("Remote Error (unknown)")
        else:
            if passedException != None:
                raise passedException
            else:
                pass

    def setCommandPid(self, newPid, exceptionData = None):
        #print 'set Command Pid called:',newPid
        self.myPid = newPid
        self.__errorFunc(newPid, exceptionData, "Error Setting Command")

    def getPid(self):
        return self.myPid

    def setKillStatus(self, status, exceptionData = None):
        self.killStatus = status
        self.__errorFunc(status, exceptionData, "Error Setting Kill Status")

    def setCommandStatus(self, status, exceptionData = None):
        self.commandStatus = status
        self.__errorFunc(status, exceptionData, "Error Setting Command Status")

    def setInputReturnStatus(self, status, exceptionData = None):
        self.inputReturnStatus = status
        self.__errorFunc(status, exceptionData, "Error Setting Input Return Status")

    def setOutputWaiting(self, isWaiting, exceptionData= None):
        self.remoteOutputWaiting = isWaiting
        self.__errorFunc(isWaiting, exceptionData, "Error Setting Output Waiting")

    def setOutput(self, output, exceptionData=None):
        #print 'setting output:',output
        self.recentRead = 1
        if self.out_fd != None and output != None:
            os.write(self.out_fd, output)
        else:
            if exceptionData != None:
                self.__errorFunc(-3, exceptionData)
            self.outputList.append(output)

    def setAutoWriteBackStatus(self, status, exceptionData=None):
        self.autoWriteBackStatus = status
        self.__errorFunc(status, exceptionData, "Error Setting Auto Writeback Status")

    def requestRead(self):
        self.proxAccess.send(self, 5, None)

    def read(self, readAll = 0):
        if readAll:
            total = ""
            for outputMsg in self.outputList:
                total = total + outputMsg
            self.outputList = []
            return total
        else:
            newRead = self.outputList[0]
            self.outputList = self.outputList[1:]
            return newRead

    def syncRead(self):
        self.recentRead = 0
        self.proxAccess.send(self, 5, None)
        timeout = 0
        while not self.recentRead and timeout < 1000:
            timeout = timeout + 1
            time.sleep(.1)
        if not self.recentRead:
            raise Exception("Connection Time Out")
        total = ""
        return self.read(1)

    def write(self, data):
        self.proxAccess.send(self, 3, data)

    def syncWrite(self, data):
        self.inputReturnStatus = -10        ## temp value
        self.proxAccess.send(self, 3, data)
        timeout = 0
        while self.inputReturnStatus == -10 and timeout < 1000:
            timeout = timeout + 1
            time.sleep(.1)
        if self.remoteOutputWaiting == -10:
            raise Exception ("Connection Time Out")
        return self.inputReturnStatus

    def kill(self):
        self.proxAccess.send(self, 1, None)

    def getStatus(self):
        self.commandStatus = -10            ## Temp Value
        self.proxAccess.send(self, 2, None)
        timeout = 0
        while self.commandStatus == -10 and timeout < 1000:
            timeout = timeout + 1
            time.sleep(.1)
        if self.commandStatus == -10:
            raise Exception ("Connection Time Out")
        return self.commandStatus

    def hasOutputWaiting(self):
        self.remoteOutputWaiting = 3        ## Temp Value
        self.proxAccess.send(self, 4, None)
        timeout = 0
        while self.remoteOutputWaiting == 3 and timeout < 1000:
            timeout = timeout + 1
            time.sleep(.1)
        if self.remoteOutputWaiting == 3:
            raise Exception ("Connection Time Out")
        return self.remoteOutputWaiting

class ShellProx:
    __verbHandler = []
    def unknown(self, data):
        return (0, data)
    __verbHandler.append(unknown)#0

    def killAckVerb(self, data):
        self.remoteKill = 'Killed'
        return (1,'!')
    __verbHandler.append(killAckVerb)#1

    def pidHandler(self, data):
        try:
            self.remotePid = int(data)
        except:
            self.remotePid = -1
        return (2,self.remotePid)
    __verbHandler.append(pidHandler)#2

    def statusHandler(self, data):
        try:
            self.remoteStatus = int(data)
        except:
            self.remoteStatus = -2
        return (3,self.remoteStatus)
    __verbHandler.append(statusHandler)#3

    def dataHandler(self, data):
        return (4,data)
    __verbHandler.append(dataHandler)#4

    def handleInputVerbs(self, data):
        try:
            sepIndex = string.index(data,",")
            dataList = (int(data[:sepIndex]),data[sepIndex+1:])
            return self.__verbHandler[int(dataList[0])](self,dataList[1])
        except:
            return self.__verbHandler[0](self,data)

    def redirectOutput(self, output_fd, input_sock):
        outputThreadStopped = 0
        while not self.FORCE_ALL_STOP and not self.remoteStatus >= 0:
            readyList, dumb1, dumb2 = select([input_sock],[],[],0)
            if len(readyList) > 0:
                data = input_sock.recv(1000)
                if len(data) == 0:
                    break
                else:
                    messageList = string.split(data,"\0")
                    for message in messageList:
                        dataList = self.handleInputVerbs(message)
                        if dataList[0] == 4:
                            os.write(output_fd,dataList[1])
                            #output_fd.flush()
        outputThreadStopped = 1
        self.internalStatus="Finished"
        input_sock.close()

    def forceInternalStop(self):
        self.FORCE_ALL_STOP=1
        time.sleep(1)
        try:
            os.write(self.__in_fd,"6,Internal Kill\n")
            os.write(self.__out_fd,"6,Internal Kill\n")
        except:
            pass

    def __tempSocketReaderHandler(self, remoteSock):
        if self.outputThreadStopped:
            readyList, d1, d2 = select([remoteSock],[],[],0)
            if len(readyList) > 0:
                data = remoteSock.recv(1000)
                messageList = string.split(data,"\0")
                for message in messageList:
                    dataList = self.handleInputVerbs(message)
                    if dataList[0] == 4:
                        os.write(output_fd,dataList[1])

    def kill(self):
        remoteSock = self.__remoteSocket
        if remoteSock == None:
            return "Not Started"
        remoteSock.send("1,!")
        waitCount= 0
        while waitCount< 10 and self.remoteKill == 'Not Killed':
            time.sleep(1)
            waitCount = waitCount+1
            self.__tempSocketReaderHandler(remoteSock)
        return self.remoteKill
            
    def __init__(self, script, output_fd = sys.stdout.fileno(), input_fd=None, hostArg=None, portArg=None, blocking=0):
        self.__in_fd = input_fd
        self.__out_fd = output_fd
        self.__remoteSocket = None
        self.FORCE_ALL_STOP=0
        self.inputThreadStopped = 1
        self.outputThreadStopped = 1
        self.remoteStatus = -1
        self.remotePid = -1
        self.internalStatus="Not Started"
        self.remoteKill = 'Not Killed'
        host = os.getenv('SHELLPROX_HOST')
        if host == None and hostArg == None:
            raise Exception("Host not set! Please set environment variable 'SHELLPROX_HOST' to remote shell host!")
        try:
            portStr = os.getenv('SHELLPROX_PORT')
            port = int(portStr)
        except:
            port = 49999    ## Default
        if hostArg != None:
            host = hostArg
        if portArg != None:
            port = portArg

        try:
            remoteShell = socket(AF_INET, SOCK_STREAM)
            remoteShell.connect((host, port))
            self.__remoteSocket = remoteShell
            argStr = script
            remoteShell.send(argStr)
            readyList, d1, d2 = select([remoteShell],[],[],10)
            if len(readyList) == 0:
                raise Exception("Connection Time Out")
            remoteShell.recv(100)   ## Get Acceptance status
            remoteShell.send("2,!") ## Get Pid
            self.internalStatus="Started"
            if input_fd != None:
                thread.start_new_thread(self.redirectInput,(input_fd,remoteShell))
            if not blocking:
                thread.start_new_thread(self.redirectOutput,(output_fd, remoteShell))
            else:
                self.redirectOutput(output_fd, remoteShell)
        except Exception, e:
            os.write(output_fd,"\nAbnormal Exit.  Shutdown may take up to twenty seconds.")
            if not blocking:
                self.outputThreadStopped = 1
            self.internalStatus = "Exception raised:"+str(e)
            try:
                self.kill(remoteShell)
                remoteShell.send("3,!")
                waitCount = 0
                while self.remoteStatus == -1 and waitCount < 10:
                    time.sleep(1)
                    waitCount = waitCount+1
                    self.__tempSocketReaderHandler(remoteShell)
            except:
                pass
            try:
                remoteShell.close()
            except:
                pass
            self.FORCE_ALL_STOP=1

    def redirectInput(self, in_fd, outSocket):
        inputThreadStopped = 0
        while not self.FORCE_ALL_STOP:
            data = os.read(in_fd,1000)
            sent = 0
            while sent < len(data):
                #outSize = os.write(out_fd,data)
                outSize = outSocket.send("4,"+data+"\0")
                sent = sent + outSize
                if outSize == 0:
                    break
        inputThreadStopped = 1


if __name__ == "__main__":
    argStr = ""
    for arg in sys.argv[1:]:
        argStr = argStr + arg + " "
    S = ShellProx(argStr, input_fd = sys.stdin.fileno(), blocking=1)
    #print "\n"+str(S.remotePid)+" terminated, status = " + str(S.remoteStatus) + "\n" + S.internalStatus
