from message_handler import *
from qt import *
import thread, time
from widgets import *
import global_vars


HostPath = global_vars.HostPath
reload_directory_time = 4000

class DirViewItem(QListViewItem):
    def __init__(self, parent, path, list_view):
        self.my_parent = parent
        self.path = path
        self.list_view = list_view
        self.basename = os.path.basename(path)
        self.type = "unknown"
        if os.path.isdir(self.path):
            self.type = "dir"
        if os.path.isfile(self.path):
            self.type = "file"
        QListViewItem.__init__(self, parent, self.basename)

    def OpenEditor(self):
        debug(5, "DirViewItem::OpenEditor : attempting to open an editor on %s" % self.path)
        if global_vars.settings.has_key("editor_path"):
            editor_path = global_vars.settings["editor_path"]
        else:
            p = self.parent
            while not isinstance(p, QWidget):
                p = p.parent
            QMessageBox.information(p, "No Editor Path", """<qt>You have not specified your editor of choice.
            Please set the editor information by selecting <b>Editor</b> from the <b>Settings</b> menu and try 
            this operation again</qt>""")
            return
        if global_vars.host == "windows":
            command = '"' + editor_path + '"' + " " + self.path
            thread.start_new_thread(self.OpenEditorThreadFunc, (command,))
        else:
            os.system(editor_path + " " + self.path + " &")

    def OpenEditorThreadFunc(self, command):
        os.system(command)
        



class DirectoryWidget(QListView):
    def __init__(self, parent, name = "Directory View"):
        QListView.__init__(self, parent, name)
        self.parent = parent
        self.nodes = {}
        self.addColumn("Name")
        self.setEnabled(0)
        self.dir = None
        QObject.connect(self, SIGNAL("rightButtonClicked ( QListViewItem *, const QPoint &, int )"), self.RightClick)
        self.file_dict = {}
        self.add_list = []
        thread.start_new_thread(self.ThreadReloadDirectory, ())

    def RightClick(self, source_node, point, column):
        #debug(5, "DirectoryWidget::RightClick called")
        if source_node == None:
            return
        popup = DirPopupMenu(self, source_node)
        popup.popup(point)

    def ThreadReloadDirectory(self):
        while 1:
            try:
                #print "thread reloading directory"
                if self.dir and os.path.exists(self.dir):
                    remove_list = self.nodes.keys()
                    remove_list.remove(self.dir)
                    add_list = self.ThreadReloadRecurse(self.dir, remove_list)
                    #print "\nin reload thread, ADD LIST is", add_list
                    #print "\nREMOVE LIST is", remove_list
                    for item in add_list:
                        self.add_list.append(item)
            except:
                pass
            #sleep for 4 seconds
            time.sleep(4)

    def ThreadReloadRecurse(self, parent_name, remove_list, max = 1):
        add_list = []
        #print "directory is", parent_name
        if max > 6:
                return add_list
        file_list = glob.glob(os.path.join(parent_name, "*"))
        #print "file list is", file_list
        #print "nodes are", self.nodes.keys()
        if len(file_list) == 0:
            return add_list
        for item in file_list:
            #if an item is not already a node
            if not item in remove_list:
                #print "adding node", item
                add_list.append((item, parent_name))
            else:
                remove_list.remove(item)
            if os.path.isdir(item):
                add_list.extend(self.ThreadReloadRecurse(item, remove_list, max + 1))
        return add_list

    def AutoReloadDirectory(self, dir):
        if dir != self.dir:
            self.RecurseClear()
            self.SetDirectory(dir)
            return
        else:
            add_list = self.add_list[:]
            self.add_list = []
            for (item, parent_name) in add_list[:]:
                if not self.nodes.has_key(item) and self.nodes.has_key(parent_name):
                    print "adding", item, parent_name
                    self.nodes[item] = DirViewItem(self.nodes[parent_name], item, self)
                    
   

    def SetDirectory(self, dir):
        self.setEnabled(1)
        self.file_dict = {}
        self.dir = dir
        self.nodes = {}
        self.nodes[self.dir] = DirViewItem(self, self.dir, self)
        self.RecurseItems(self.dir)
        self.nodes[self.dir].setOpen(1)
        
    
    def RecurseClear(self, node = "root"):
        flag = 0
        if node == "root":
            try:
                flag = 1
                node = self.nodes[self.dir]
            except:
                return
        if node.childCount() > 0:
            child = node.firstChild()
            if child != None:
                next_sib = 1
                while next_sib != None:
                    next_sib = child.nextSibling()
                    self.RecurseClear(child)
                    node.takeItem(child)
                    del(child)
                    child = next_sib

        if flag:
            self.takeItem(self.nodes[self.dir])
            del(self.nodes[self.dir])

    def RecurseItems(self, parent_name, max = 1):
        if max > 6:
            return
        file_list = glob.glob(os.path.join(parent_name, "*"))
        if len(file_list) == 0:
            return
        for item in file_list:
            self.file_dict[item] = 1
            self.nodes[item] = DirViewItem(self.nodes[parent_name], item, self)
            self.RecurseItems(item, max + 1)


class DirPopupMenu(QPopupMenu, HandlerChild):
    def __init__(self, parent, dir_node):
        HandlerChild.__init__(self, "dir_popup_menu", temporary=1)
        QPopupMenu.__init__(self, parent, "Source File Popup Menu")
        self.parent = parent
        self.dir_node = dir_node
        self.path = dir_node.path
        self.make_section_mapping = {}
        self.make_section_lines = {}
        if self.dir_node.type == "dir":
            self.insertItem("&Add a file to this directory", self.AddFile)
            self.insertItem("&Add a Directory to this directory", self.AddDir)

        if self.dir_node.type == "file":
            self.insertItem("&Open this file in an editor", self.dir_node.OpenEditor)
            self.insertItem("&Delete this file", self.DeleteFile)
            self.insertItem("&Save this file", self.parent.parent.SaveFile)
            if self.dir_node.basename.find("akefile") != -1:
                make_sections = self.GetMakeSections()
                make_sections.sort()
                self.make_popup = QPopupMenu(self, "make section popup")
                QObject.connect(self.make_popup, SIGNAL("activated(int)"), self.GotoMakeSection)
                for section in make_sections:
                    self.make_section_mapping[self.make_popup.insertItem(section)] = section
                self.insertItem("Go to &Makefile Section", self.make_popup, )

    def GotoMakeSection(self, item_id):
        section = self.make_section_mapping[item_id]
        debug(5, "GotoMakeSection: %s selected" % section)
        line = self.make_section_lines[section]
        self.SendMessage("source_file_editor", "SetCursorPosition", [line, 0])

    def GetMakeSections(self):
        text = self.EmitMessage("GetSourceText")
        if text == None:
            debug(5, "DirPopupMenu::GetMakeSection : Could not get source test")
            return []
        sections = []
        lines = text.split("\n")
        for i in range(len(lines)):
            if len(lines[i]) == 0 or lines[i][0] in " \t#" :
                continue
            if lines[i].find(":") != -1:
                section_name = lines[i][0:lines[i].find(":")].strip()
                sections.append(section_name)
                self.make_section_lines[section_name] = i

        debug(5, "found sections", sections)
        return sections


    def DeleteFile(self):
        debug(5, "DirNodePopupMenu::DeleteFile : called on %s" % self.path)
        answer = QMessageBox.warning(self, "Delete File", """<qt>Are you sure that you want to delete <b>%s</b>?
                You will not be able to recover this file if you proceed!""" % os.path.basename(self.dir_node.path),
                "&Yes, Please Delete", "&No, Do Not Delete", "", 0, 1)
        if answer == 1:
            return

        if not os.path.exists(self.dir_node.path):
            debug(5, "DirNodePopupMenu::DeleteFile : called on %s, but it does not exist" % self.path)
        status = log.ExecuteCommand("rm -f %s" % global_vars.BuilderPath(self.dir_node.path))
        if status:
            debug(5, "DirNodePopupMenu::DeleteFile : delete failed" % self.path)

        parent = self.dir_node.parent
        parent.takeItem(self.dir_node)
        del(self.dir_node)

    def AddFile(self):
        pass

    def AddDir(self):
        pass

    def Dummy(self):
        debug(5, "DirPopupMenu::Dummy %s" % self.path)


class PatchDialog(QDialog):
    def __init__(self, parent):
        QDialog.__init__(self, parent, "Patch Dialog", 1)

        self.setCaption("Make Patch")

        self.vl1 = QVBoxLayout(self, 10, 10)

        self.grid = QGridLayout(self.vl1, 2, 2, 10, "grid layout")
        self.name_label = QLabel("Patch filename: ", self)
        self.grid.addWidget(self.name_label, 0, 0)
        self.name_entry = QLineEdit(self, "")
        self.grid.addWidget(self.name_entry, 0, 1)
        self.lbc_label = QLabel("Add patch to lbc: ", self)
        self.grid.addWidget(self.lbc_label, 1, 0)
        self.lbc_checkbox = QCheckBox(self, "lbc checkbox")
        self.lbc_checkbox.setChecked(1)
        self.grid.addWidget(self.lbc_checkbox, 1, 1)

        self.hl3 = QHBoxLayout(self.vl1, 10)
        self.patch_button = QPushButton("Make &Patch", self)
        QObject.connect(self.patch_button, SIGNAL("clicked()"), self.MakePatch)
        self.hl3.addWidget(self.patch_button)
        self.cancel_button = QPushButton("&Cancel", self)
        QObject.connect(self.cancel_button, SIGNAL("clicked()"), self.Cancel)
        self.hl3.addWidget(self.cancel_button)

        self.vl1.activate
        self.status = -1
        self.patch_name = ""

    def GetStatus(self):
        return self.status

    def Cancel(self):
        self.patch_name = ""
        self.status = 0
        self.reject()

    def MakePatch(self):
        self.patch_name = str(self.name_entry.text())
        if not global_vars.IsValidFilename(self.patch_name):
            QMessageBox.warning(self, "Bad Filename", """<qt> The filename that you entered for
            the patch filename is invalid.  Please enter a valid filename
            (do not include directory in the name.)""", QMessageBox.Ok, 0, 0)
            return
        self.status = 1
        self.accept()

    def GetPatchName(self):
        return self.patch_name

    def GetLBCStatus(self):
        if self.lbc_checkbox.isChecked():
            return 1
        else:
            return 0

class SourceEditor(QMultiLineEdit, HandlerChild):
    def __init__(self, parent, name = "SourceEditor"):
        QMultiLineEdit.__init__(self, parent, name)
        HandlerChild.__init__(self, "source_file_editor")
        self.SetListener("GetSourceText", self.GetSourceText)
        self.SetListener("SetCursorPosition", self.SetPosition)
        self.installEventFilter(self)
        self.fixed_font = QFont("courier", self.font().pointSize())
        self.fixed_font.setFixedPitch(1)
        self.setFont(self.fixed_font)

    def GetSourceText(self):
        return str(self.text())

    def SetPosition(self, row, column):
        self.setFocus()
        self.setCursorPosition(row, column)

    #We'll catch tab keypresses and send a tab to the text area instead
    #of moveing to the next widget as the tab key usually does
    def eventFilter(self, object, event):
        if event.type() == QEvent.KeyPress:
            if event.key() == event.Key_Tab:
                self.insert("\t")
            else:
                self.keyPressEvent(event)
            return 1
        return 0



class SourceFrame(QFrame):
    def __init__(self, parent, main):
        global debug, log, handler
        debug = global_vars.get("debug")
        log = global_vars.get("log")
        handler = global_vars.get("handler")
        self.main = main
        QFrame.__init__(self, parent, "Source Frame")

        self.hl1 = QHBoxLayout(self, 10, 10)
        self.dir_widget = DirectoryWidget(self)
        self.hl1.addWidget(self.dir_widget, 1)

        self.vl1 = QVBoxLayout()
        self.hl1.addLayout(self.vl1, 2)

        self.edit_label = QLabel("Filename", self)
        self.vl1.addWidget(self.edit_label, 0)
        self.editor = SourceEditor(self)
        self.vl1.addWidget(self.editor, 1)

        self.hl1.activate()

        QObject.connect(self.dir_widget, SIGNAL("selectionChanged(QListViewItem *)"), self.ShowFile)
        self.in_use = 0
        self.log = None
        self.setEnabled(0)
        self.cur_filename = ""
        self.diff_mode = 0
        self.selected_item = None

        self.auto_reload_timer = QTimer(self, "source reload timer")
        QObject.connect(self.auto_reload_timer, SIGNAL("timeout()"), self.AutoReloadDirectory)
        self.auto_reload_timer.start(500)

    def Close(self):
        if self.in_use and self.editor.edited():
            debug(5, "SourceFrame::SetDiffMode : %s has been edited" % self.selected_item.path)
            answer = QMessageBox.warning(self, "File Changed", """<qt>The file <b>%s</b> has changed.
                Would you like to save it before closing?""" % os.path.basename(self.selected_item.path),
                "&Yes", "&No", "", 0, 1)
            if answer == 0:
                self.SaveFile()
        self.dir_widget.RecurseClear()
        self.editor.setText("")
        self.in_use = 0
        self.setEnabled(0)


    def SetDiffMode(self, diff_mode):
        if diff_mode == 1 and self.diff_mode == 0 and self.in_use and self.editor.edited():
            debug(5, "SourceFrame::SetDiffMode : %s has been edited" % self.selected_item.path)
            answer = QMessageBox.warning(self, "File Changed", """<qt>The file <b>%s</b> has changed.
                Would you like to save it before viewing the diff?""" % os.path.basename(self.selected_item.path),
                "&Yes", "&No", "", 0, 1)
            if answer == 0:
                self.SaveFile()

        self.diff_mode = diff_mode
        self.editor.setReadOnly(diff_mode)
        self.LoadFile()

    def RegisterLogger(self, log):
        self.log = log

    def SendText(self, text):
        if self.log == None:
            Debug(5, "SourceFrame::SendText : No logger Registered")
        else:
            self.log.SendText(text)

    def GetBuildPath(self, path):
        projectdir = self.main.GetProjDir()
        if not projectdir:
            debug(5, "SourceFrame::MakePatch : missing projectdir")
            return
        builddir = self.main.GetBuildDir()
        if not builddir:
            debug(5, "SourceFrame::MakePatch : missing builddir")
            return
        base = os.path.commonprefix([os.path.join(projectdir, "build/rpmdir/BUILD", builddir), path])
        i = len(base)
        buildpath = path[i:]
        while buildpath and buildpath[0] == "/":
            buildpath = buildpath[1:]
        return buildpath


    def MakePatch(self):
        self.patch_dialog = PatchDialog(self)
        self.patch_dialog.show()
        status = self.patch_dialog.GetStatus()
        if status != 1:
            return
        patch_name = self.patch_dialog.GetPatchName()
        lbc = self.patch_dialog.GetLBCStatus()

        self.SendText("\n==================== Making Patch ===================\n\n")
        package_name = self.main.GetPackageName()
        if not package_name:
            debug(5, "SourceFrame::MakePatch : missing package_name")
            return
        projectdir = global_vars.BuilderPath(self.main.GetProjDir())
        if not projectdir:
            debug(5, "SourceFrame::MakePatch : missing projectdir")
            return
        builddir = global_vars.BuilderPath(self.main.GetBuildDir())
        if not builddir:
            debug(5, "SourceFrame::MakePatch : missing builddir")
            return

        builddir = builddir.split("/")[0]

        command = """
        cd %(projectdir)s/build/rpmdir/BUILD
        if [ ! -d "%(builddir)s" ] ; then exit 2; fi
        if [ ! -d "%(builddir)s.orig" ] ; then exit 2; fi
        /usr/bin/twcl --projdir %(projectdir)s --pkg %(package_name)s --batch clean
        diff -rc --new-file %(builddir)s.orig %(builddir)s > %(projectdir)s/Packages/local/%(patch_name)s
        """ % vars()
        debug(5, "======== script ======", command, "\n\n\n")
        command_status = self.log.ExecuteCommand(command)
        if command_status > 1:
            self.SendText("===== Error While Making Patch =====")
            return
        if command_status == 0:
            self.SendText("===== No Differences Found =====")
            return
        if command_status == 1:
            self.SendText("===== Finished Making Patch =====")
            
        if lbc:
            patches = self.main.lbc_frame.GetField("patches").strip()
            patches = patches + "\n" + patch_name
            patches = patches.strip()
            self.main.lbc_frame.SetField("local", "patches", patches)
            self.main.WriteLBC()

    def PreparePackageSource(self):
        package_name = self.main.GetPackageName()
        if not package_name:
            debug(5, "SourceFrame::ModifiedPackageSource : missing package_name")
            return
        projectdir = global_vars.BuilderPath(self.main.GetProjDir())
        if not projectdir:
            debug(5, "SourceFrame::ModifiedPackageSource : missing projectdir")
            return
        builddir = self.main.GetBuildDir()
        builddir = builddir.split("/")[0]
        if not builddir:
            debug(5, "SourceFrame::ModifiedPackageSource : missing builddir")
            QMessageBox.warning(self, "No Build Directory", """<qt> There is currently no build
            directory specified for this package.  Please enter a build directory in the LBC and
            select this option again.  You may select <b>Determine Build Directory</b> from the <b>LBC</b>
            menu to have the packager determine it for you.</qt>""")
            return
        path = global_vars.HostPath(os.path.join(projectdir, "build/rpmdir/BUILD", builddir))
        if os.path.exists(path) or os.path.exists(path + ".orig"):
            answer = QMessageBox.warning(self, "Unpackage Message", """<qt>
                The Packager will now set up a directory structure to be used while working with source for %(package_name)s.
                The following steps will occur:
                <ol>
                <li> If the sources for %(package_name)s have not been unpackaged, they will be.  If they 
                have already been unpackaged no changes will occur to the working source.
                <li> The %(builddir)s.orig directory will be removed if it exists</li>
                <li> The %(package_name)s tarball will be untarred and moved to %(builddir)s.orig for patches later</li>
                </ol>
                When this is done, a clean version the contents of the package's tarball with patches applied will
                be located at %(builddir)s.orig in the src directory of your project.
                Are you sure that you would like to proceed?</qt>""" % vars(), "&No", "&Yes", "", 1, 0)
            if answer == 0:
                return
        
        wait_box = WaitBox(self, "Preparing Source.  Please wait...", "Source");
        wait_box.show()
        self.SendText("\n======Creating %s.orig directory for patches later========\n\n" % builddir)
        command = """
        #erase all old directories first
        cd %(projectdir)s/build/rpmdir/BUILD
        if [ -e "%(builddir)s.orig" ] ; then rm -rf %(builddir)s.orig; fi
        #move the current directory to .work temporarliy
        if [ ! -e "%(builddir)s" ] ; then /usr/bin/twcl --projdir %(projectdir)s --pkg %(package_name)s --batch --force --nuke prepare ; fi 
        mv -f %(builddir)s %(builddir)s.work
        #move the package state so the builder thinks we're starting from scratch
        if [ -e "%(projectdir)s/build/packagestate/%(package_name)s.st" ] ;
        then
            mv -f %(projectdir)s/build/packagestate/%(package_name)s.st %(projectdir)s/build/packagestate/%(package_name)s.work
        fi
        #explode out the source and then move it to .orig
        /usr/bin/twcl --projdir %(projectdir)s --pkg %(package_name)s --batch --force --nuke prepare
        #move the just built dir to dir.work and make a .orig copy for patches later
        mv -f %(builddir)s %(builddir)s.orig
        #put the package stat back in
        if [ -e "%(projectdir)s/build/packagestate/%(package_name)s.work" ] ;
        then
            mv -f %(projectdir)s/build/packagestate/%(package_name)s.work %(projectdir)s/build/packagestate/%(package_name)s.st
        fi
        mv -f %(builddir)s.work %(builddir)s
        """ % vars()
        self.log.ExecuteCommand(command)
        wait_box.hide()
        self.OpenPackage()
    
    def LoadFile(self):
        self.editor.setText("")
        if self.selected_item == None:
            debug(5, "SourceFrame::LoadFile : there is no file selected")
            return
        if self.selected_item.type == "unknown":
            debug(5, "%s if of unknown type!" % self.selected_item_path)
            return
        if not os.path.exists(self.selected_item.path):
            debug(5, "%s does not exist" % self.selected_item.path)
            return
        if self.diff_mode:
            builddir = global_vars.BuilderPath(self.main.GetBuildDir())
            if not builddir:
                debug(5, "SourceFrame::LoadFile : missing builddir")
                return
            projectdir = global_vars.BuilderPath(self.main.GetProjDir())
            if not projectdir:
                debug(5, "SourceFrame::LoadFile : missing projectdir")
                return
            basename = self.GetBuildPath(self.selected_item.path)
            if self.selected_item.type == "file":
                command = "cd %(projectdir)s/build/rpmdir/BUILD/; diff -c --new-file %(builddir)s.orig/%(basename)s %(builddir)s/%(basename)s" % vars()
            if self.selected_item.type == "dir":
                command = "cd %(projectdir)s/build/rpmdir/BUILD/; diff -c --new-file %(builddir)s.orig/%(basename)s %(builddir)s/%(basename)s" % vars()
            (status, diff) = self.log.ExecuteCommand(command, 1, 0)
            #print diff
            if status == 0:
                self.editor.setText("No differences found")
            else:
                #diff = re.sub("\\+.*\n", "", diff)
                self.editor.setText(diff)
            self.in_use = 0
        else:
            if self.selected_item.type == "file":
                try:
                    fd = open(self.selected_item.path)
                    file_buf = fd.read()
                    fd.close()
                except:
                    debug(5, "SourceFrame::LoadFile : error opening", self.selected_item.path)
                    self.editor.setEdited(0)
                    self.in_use = 0
                    return
                self.editor.setText(file_buf)
                self.in_use = 1
                self.editor.setReadOnly(0)
            if self.selected_item.type == "dir":
                command = "cd %s; ls -l --color=never" % global_vars.BuilderPath(self.selected_item.path)
                (status, text) = self.log.ExecuteCommand(command, 1, 0)
                text = re.sub("\\+.*\n", "", text)
                self.editor.setText(text)
                self.editor.setReadOnly(1)
        self.edit_label.setText(os.path.basename(self.selected_item.path))
        self.editor.setEdited(0)


    def SaveFile(self):
        try:
            debug(5, "SourceFrame::SaveFile : writing", self.cur_filename)
            file_text = str(self.editor.text())
            log.ExecuteCommand("chmod u+w " + global_vars.BuilderPath(self.selected_item.path))
            fd = open(self.selected_item.path, "w")
            fd.write(file_text)
            fd.close()
            self.editor.setEdited(0)
            return 1
        except:
            debug(5, "SourceFrame::SaveFile : error writing", self.cur_filename)
            return 0


    def ShowFile(self, selected_item):
        self.old_item = self.selected_item
        if selected_item == None:
            return
        if self.old_item == selected_item:
            debug(5, "SourceFrame::ShowFile : selection has not changed")
            return

        if self.in_use and self.editor.edited():
            debug(5, "SourceFrame::ShowFile : %s has been edited" % self.selected_item.path)
            answer = QMessageBox.warning(self, "File Changed", """<qt>The file <b>%s</b> has changed.
                Would you like to save it before continuing?""" % os.path.basename(self.selected_item.path),
                "&Yes", "&No", "", 0, 1)
            if answer == 2:
                self.dir_widget.setSelected(self.selected_item, 1)
            if answer == 0:
                self.SaveFile()
        debug(5, "Selection Changed", selected_item.path)
        self.selected_item = selected_item
        if os.path.exists(selected_item.path):
               self.LoadFile()


    def OpenPackage(self, interactive=0):
        self.setEnabled(1)
        self.cur_filename = ""
        debug(5, "SourceFrame::OpenPackage : being called")
        self.dir_widget.RecurseClear()
        build_dir = self.main.GetBuildDir()
        projectdir = self.main.GetProjDir()
        
        build_dir = build_dir.split("/")[0]
        if build_dir.strip() == "":
            source_file = self.main.GetSourceFile()
            matches = re.findall(r"([^/]*)-", source_file)
            if len(matches):
                build_dir = matches[0]
        
        if not build_dir:
            if interactive:
                QMessageBox.warning(self, "No Build Directory", """<qt>The build 
                directory for this package is not available.  Please fill in the 
                <b>Build Directory Name</b> in the LBC or select <b>Determine Build
                Directory From Source</b> from the LBC menu and select this option
                again.</qt>""")
            return
        if not projectdir:
            debug(5, "SourceFrame::OpenPackage : missing build_dir or projectdir")
            return

        path = os.path.join(projectdir, "build/rpmdir/BUILD", build_dir)
        debug(5, "SourceFrame::OpenPackage : path is", path)
        if os.path.exists(path):
            debug(5, "SourceFrame::OpenPackage : getting files")
            self.dir_widget.setEnabled(1)
            self.dir_widget.SetDirectory(path)
        else:
            debug(5, "SourceFrame::OpenPackage : no files to get")
        self.selected_item = None
       
    
    
    def AutoReloadDirectory(self):
        build_dir = self.main.GetBuildDir()
        build_dir = build_dir.split("/")[0]
        projectdir = self.main.GetProjDir()
        
        #if we don't know the build_dir then try to guess it
        if build_dir.strip() == "":
            source_file = self.main.GetSourceFile()
            matches = re.findall(r"([^/]*)-", source_file)
            if len(matches):
                build_dir = matches[0]
        
        if not build_dir.strip() or not projectdir.strip():
            return
        path = os.path.join(projectdir, "build/rpmdir/BUILD", build_dir)
        if os.path.exists(path):
            self.dir_widget.AutoReloadDirectory(path)
        
    def LoadDirectory(self, project_directory, build_directory, interactive = 0):
        if not build_dir:
            QMessageBox.warning(self, "No Build Directory", """<qt>The build 
            directory for this package is not available.  Please fill in the 
            <b>Build Directory Name</b> in the LBC or select <b>Determine Build
            Directory From Source</b> from the LBC menu and select this option
            again.</qt>""")
            return
        if not projectdir:
            debug(5, "SourceFrame::OpenPackage : missing build_dir or projectdir")
            return

        path = os.path.join(projectdir, "build/rpmdir/BUILD", build_dir)
        debug(5, "SourceFrame::OpenPackage : path is", path)
        if os.path.exists(path):
            debug(5, "SourceFrame::OpenPackage : getting files")
            self.dir_widget.setEnabled(1)
            self.dir_widget.SetDirectory(path)
        else:
            debug(5, "SourceFrame::OpenPackage : no files to get")
        self.selected_item = None
        

