from qt import *
import os, glob, commands


class WaitBox(QDialog):
    def __init__(self, parent, message, title="Please Wait"):
        QDialog.__init__(self, parent, "wait dialog box for" + message)
        self.setCaption(title)
        self.vl1 = QVBoxLayout(self, 10)
        self.l = QLabel(message, self)
        self.vl1.addWidget(self.l)            
        self.vl1.activate()

class ColorableLabel(QLabel):
    def __init__(self, a_string, parent, name="", flags = 0):
        QLabel.__init__(self, a_string, parent, name, flags)
        self.bold = 0
        self.color = QColor(0, 0, 0)
        self.my_palette = QPalette(self.palette())

    def setColor(self, r, g=-1, b=-1):
        if b == -1:
            self.color = QColor(r)
        else:
            print "new color is %d, %d, %d" % (r, g, b)
            self.color = QColor(r, g, b)
        self.acg = QColorGroup(self.my_palette.active())
        self.acg.setColor(QColorGroup.Foreground, self.color)
        self.my_palette.setActive(self.acg)
        self.my_palette.setInactive(self.acg)

        self.setPalette(self.my_palette)
        self.repaint()
        
    def setBold(self, bold):
        if not bold in [0, 1]:
            debug(5, "ColorableLabel:: setBold : received bad bols value %s" % str(bold))
            return
        self.bold = bold
        self.repaint()

    def bold(self):
        return self.bold

    def drawContents(self, painter):
        font = painter.font()
        font.setBold(self.bold)
        QLabel.drawContents(self, painter)




class ColorableRadioButton(QRadioButton):
    i = 0
    def __init__(self, parent, name):
        QRadioButton.__init__(self, parent, name)
        self.bold = 0 #by default the 
        self.color = QColor(0, 0, 0) #The default color is black
        self.my_palette = QPalette(self.palette())
        
    
    def setColor(self, r, g=-1, b=-1):
        if b == -1:
            self.color = QColor(r)
        else:
            print "new color is %d, %d, %d" % (r, g, b)
            self.color = QColor(r, g, b)
        self.acg = QColorGroup(self.my_palette.active())
        self.acg.setColor(QColorGroup.Foreground, self.color)
        self.my_palette.setActive(self.acg)
        self.my_palette.setInactive(self.acg)
        self.setPalette(self.my_palette)
        self.repaint()
    
    def setBold(self, bold):
        if not bold in [0, 1]:
            debug(5, "BoldableRadioButton:: setBold : received bad bols value %s" % str(bold))
            return
        self.bold = bold
        self.repaint()

    def bold(self):
        return self.bold

    def drawButtonLabel(self, painter):
        font = painter.font()
        font.setBold(self.bold)
        QRadioButton.drawButtonLabel(self, painter)

class OutputDialog(QDialog):
    def __init__(self, parent, text, name="", modal=0, flags = 0):
        QDialog.__init__(self, parent, name, modal, flags)
        self.setCaption(name)
        self.ok_button = QPushButton("OK", self, "ok button")
        QObject.connect(self.ok_button, SIGNAL("clicked()"), self.accept)
        self.box = QMultiLineEdit(self, "multiline edit")
        self.vlayout = QVBoxLayout(self, 10, 10)
        self.label = QLabel(text, self)
        self.vlayout.addWidget(self.label)
        self.vlayout.addWidget(self.box)
        self.hlayout = QHBoxLayout(self.vlayout, 10)
        self.hlayout.addStretch(1)
        self.hlayout.add(self.ok_button)
        self.hlayout.addStretch(1)
        
        self.resize(300, 400) 
        self.fixed_font = QFont("courier", self.box.font().pointSize())
        self.fixed_font.setFixedPitch(1)
        self.box.setFont(self.fixed_font)

    def SetText(self, text):
        self.box.setText(text)

    def AddText(self, text):
        self.box.insertLine(text) 




